@@ -1,9 +1,9 @@
-static const char *ClassId    = "$Id: DataBaseClass.cpp,v 2.38 2009/04/06 13:11:08 pascal_verdier Exp $";
+static const char *ClassId    = "$Id: DataBaseClass.cpp,v 2.39 2010/09/21 11:43:21 taurel Exp $";
 static const char *CvsPath    = "$Source: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v $";
 static const char *SvnPath    = "$HeadURL: $";
-static const char *TagName    = "$Name: DataBase-Release_4_5 $";
+static const char *TagName    = "$Name: Database-Release_4_8 $";
 static const char *HttpServer = "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.38 2009/04/06 13:11:08 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.39 2010/09/21 11:43:21 taurel Exp $";
 //+=============================================================================
 //
 // file :        DataBaseClass.cpp
@@ -17,9 +17,32 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 //
 // author(s) :   A.Gotz, JL Pons, P Verdier
 //
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
 // $Version$
 //
 // $Log: DataBaseClass.cpp,v $
+// Revision 2.39  2010/09/21 11:43:21  taurel
+// - Add GPL stuff
+//
 // Revision 2.38  2009/04/06 13:11:08  pascal_verdier
 // DbMySqlSelect command added.
 //
@@ -195,10 +218,6 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 // converted DataBaseds to POGO
 //
 //
-// copyleft :   European Synchrotron Radiation Facility
-//              BP 220, Grenoble 38043
-//              FRANCE
-//
 //-=============================================================================
 //
 //  This file was generated by POGO
