@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/CheckProcessUtil.cpp,v 3.11 2008/12/12 13:30:44 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/CheckProcessUtil.cpp,v 3.15 2010/10/15 06:20:32 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         CheckProcessUtil.cpp
@@ -9,9 +9,42 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/C
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.11 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.15 $
 //
 // $Log: CheckProcessUtil.cpp,v $
+// Revision 3.15  2010/10/15 06:20:32  pascal_verdier
+// Copyright added.
+//
+// Revision 3.14  2010/09/21 12:18:57  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.13  2010/08/09 06:52:25  pascal_verdier
+// Patch for Python module added (thanks to Tiago).
+//
+// Revision 3.12  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.11  2008/12/12 13:30:44  pascal_verdier
 // Problem on python server options fixed.
 //
@@ -47,11 +80,6 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/C
 // Revision 3.1  2008/02/29 15:15:05  pascal_verdier
 // Checking running processes by system call added.
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
 //-=============================================================================
 
 
@@ -59,7 +87,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/C
 #include <CheckProcessUtil.h>
 
 #ifndef	TIME_VAR
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 
 #	define	TimeVal	struct timeval
 #	define	GetTime(t)	gettimeofday(&t, NULL);
@@ -74,7 +102,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/C
 #	define	Elapsed(before, after)	\
 		1000*(after.time - before.time) + (after.millitm - before.millitm)
 
-#endif	/*	WIN32		*/
+#endif	/*	_TG_WINDOWS_		*/
 #endif	/*	TIME_VAR	*/
 
 
@@ -84,11 +112,11 @@ namespace Starter_ns
 //=============================================================
 ProcessData::ProcessData()
 {
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	//	Under win 2000 or before the process name is shorted to 15 char
 	//	If win 2000, take it from command line.
 	win2000 = isWin2000();
-#endif	/*	WIN32		*/
+#endif	/*	_TG_WINDOWS_		*/
 }
 //=============================================================
 //=============================================================
@@ -100,7 +128,7 @@ ProcessData::~ProcessData()
 	proc_list.clear();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //=============================================================
 //=============================================================
 string  ProcessData::parseNameFromCmdLine(string name, string cmdline)
@@ -359,7 +387,7 @@ string ProcessData::errorCodeToString (DWORD err_code,  string src)
 
 
 
-#else	//	WIN32
+#else	//	_TG_WINDOWS_
 
 
 
@@ -571,7 +599,7 @@ bool  ProcessData::manageProcFiles(Process *process)
 	return true;
 #endif
 }
-#endif	//	WIN32
+#endif	//	_TG_WINDOWS_
 
 
 
@@ -584,7 +612,7 @@ bool  ProcessData::manageProcFiles(Process *process)
 void ProcessData::check_cpp_process(Process* process)
 {
 	//	Remove path
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	if (process->line_args.size()==0)
 		process->name = "";
 	else
@@ -603,7 +631,7 @@ bool ProcessData::check_java_process(Process* process)
 {
 	if (process->line_args.size()==0)
 		return false;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (process->name!="java" &&
 		process->name!="javaw")
 		return false;
@@ -647,7 +675,7 @@ bool ProcessData::check_python_process(Process* process)
 {
 	if (process->line_args.size()==0)
 		return false;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (process->name!="python")
 		return false;
 #else
@@ -659,28 +687,49 @@ bool ProcessData::check_python_process(Process* process)
 		return false;	//	No module loaded
 
 	//	To get python module name
-	bool	found = false;
-	int		args_idx = 2;
-	for (int i=1 ; !found &&  i<process->line_args.size()-1 ; i++)
+	bool found_py_module = false;
+	int args_idx = 1;
+	for (; args_idx < process->line_args.size()-1 ; args_idx++)
 	{
-		//	Do not check if python option
-		if (process->line_args[i]!="" && process->line_args[i].c_str()[0]!='-')
-		{
-			string	full_name(process->line_args[i]);
-			string::size_type	start = 0;
-			string::size_type	end;
-			while ((end=full_name.find('/', start))!=string::npos)
-				start = end+1;
-			while ((end=full_name.find('\\', start))!=string::npos)
-				start = end+1;
+	    const string &arg = process->line_args[args_idx];
+	    
+	    if (arg.size() == 0)
+	        continue;
+	    
+	    if (arg[0] == '-')
+	    {
+	        // special python arguments that receive and additional parameter
+	        if (arg.size() > 1 && (arg[1] == 'Q' || arg[1] == 'W'))
+	            args_idx++; 
+	        continue;
+	    }
+	    
+	    // we reached the python file in execution
+        found_py_module = true;
+	    
+        string::size_type start = arg.rfind('/');
+        if (start == string::npos)
+            start = arg.rfind('\\');
+        
+        if (start == string::npos)
+            start = 0;
+        else
+            start++;
 
-			//	Get last one
-			end = full_name.find(".py", start);
-			process->name = full_name.substr(start, (end-start));
-			args_idx = i+1;
-		}
+	    string::size_type end = arg.rfind(".py");
+        
+        if (end == string::npos)
+            process->name = arg.substr(start);
+        else
+            process->name = arg.substr(start, end-start);
+        // everything from now on is an argument
+        args_idx++;
+        break;
 	}
 
+    if (!found_py_module)
+        return false;
+
 	for (int i=args_idx ; i<process->line_args.size() ; i++)
 		process->proc_args.push_back(process->line_args[i]);
 	return true;
@@ -709,7 +758,7 @@ void ProcessData::build_server_names(Process* process)
 		transform(instance.begin(), instance.end(),
 				instance.begin(), ::tolower);
 		process->servname += instance;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	//	Wain32 is case unsensitive
 	transform(process->servname.begin(), process->servname.end(),
 					process->servname.begin(), ::tolower);
@@ -792,7 +841,7 @@ void ProcessData::update_process_list()
 			process->line_args.push_back(s);
 
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		if (process->line_args.size()>0)
 			process->name = process->line_args[0];
 		else
@@ -930,7 +979,7 @@ bool CheckProcessUtil::is_server_running(string argin)
 		return false;	//	Not a server name
 	pos++;
 	string	dsname(argin.substr(0, pos));
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	//	Wain32 is case unsensitive
 	transform(dsname.begin(), dsname.end(),
 					dsname.begin(), ::tolower);
@@ -987,3 +1036,6 @@ void *CheckProcessUtil::run_undetached(void *ptr)
 
 
 }	//	namespace
+
+
+ 	  	 
