@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/PingThread.cpp,v 1.7 2008/12/16 10:44:57 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/PingThread.cpp,v 1.11 2010/10/15 06:20:33 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         StarterUtil.cpp
@@ -9,9 +9,42 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/P
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 1.7 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 1.11 $
 //
 // $Log: PingThread.cpp,v $
+// Revision 1.11  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 1.10  2010/10/08 08:48:50  pascal_verdier
+// Include files order changed.
+//
+// Revision 1.9  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 1.8  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 1.7  2008/12/16 10:44:57  pascal_verdier
 // Memory leak fixed.
 //
@@ -33,31 +66,26 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/P
 // Revision 1.1  2006/02/09 12:00:15  pascal_verdier
 // A ping thread is now started for each server.
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
 //-=============================================================================
 
-#include <stdio.h>
+#include <tango.h>
 
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 #	include <sys/time.h>
 #endif
 
 
-#include <tango.h>
 #include <sstream>
 #include <PingThread.h>
 
 #ifndef	TIME_VAR
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 
 #	define	TimeVal	struct timeval
 #	define	GetTime(t)	gettimeofday(&t, NULL);
@@ -72,7 +100,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/P
 #	define	Elapsed(before, after)	\
 		1000*(after.time - before.time) + (after.millitm - before.millitm)
 
-#endif	/*	WIN32		*/
+#endif	/*	_TG_WINDOWS_		*/
 #endif	/*	TIME_VAR	*/
 
 namespace Starter_ns
