@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.17 2008/12/12 13:29:56 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.22 2010/10/15 06:20:33 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -9,9 +9,45 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.17 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.22 $
 //
 // $Log: StartProcessThread.cpp,v $
+// Revision 3.22  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.21  2010/10/08 08:48:50  pascal_verdier
+// Include files order changed.
+//
+// Revision 3.20  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.19  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
+// Revision 3.18  2010/02/09 10:25:38  pascal_verdier
+// __MACOS__ compiler flag added.
+//
 // Revision 3.17  2008/12/12 13:29:56  pascal_verdier
 // Log in file start and stop for servers and itself.
 //
@@ -67,12 +103,13 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 // Revision 3.1  2006/04/24 07:06:27  pascal_verdier
 // A thread is started for each level when at servers startup.
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
-#ifdef WIN32
+//+=============================================================================
+
+
+#include <tango.h>
+
+
+#ifdef _TG_WINDOWS_
 #	include <process.h>
 #	include <direct.h>
 #	include <io.h>
@@ -83,7 +120,6 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 
 #include <fcntl.h>
 
-#include <tango.h>
 #include <Starter.h>
 
 namespace Starter_ns
@@ -142,7 +178,7 @@ void *StartProcessThread::run_undetached(void *ptr)
 		Tango::DeviceProxy	*serv;
 		while (failed==false && started==false && (t1-t0)<starter->serverStartupTimeout)
 		{
-#		ifdef WIN32
+#		ifdef _TG_WINDOWS_
 			_sleep(1000);
 #		else
 			sleep(1);
@@ -217,7 +253,7 @@ void *StartProcessThread::run_undetached(void *ptr)
 	starter->start_proc_data->remove_current_level();
 	return NULL;
 }
-#ifndef	WIN32
+#ifndef	_TG_WINDOWS_
 //+------------------------------------------------------------------
 /**
  *	Start one process
@@ -248,8 +284,14 @@ void StartProcessThread::start_process(NewProcess *process)
 		case 0:
 			{
 				// Change process group and close control tty
-#if (defined __darwin__ || defined __freebsd__)
+#if defined(__darwin__) || defined( __MACOS__)
+
+				setpgrp();
+
+#elif defined (__freebsd__)
+
 				setpgrp(0,setsid());
+
 #else										
 				setpgrp();
 
@@ -293,7 +335,7 @@ void StartProcessThread::start_process(NewProcess *process)
 }
 
 
-#else	//	WIN32
+#else	//	_TG_WINDOWS_
 
 
 //+----------------------------------------------------------
@@ -312,6 +354,35 @@ void StartProcessThread::start_process(NewProcess *process)
 	win_thread->start();
 }
 
+//+------------------------------------------------------------------
+/**
+ *	Set the path between cotes for windows.
+ *
+ *      @param  servname	server name
+ */
+//+------------------------------------------------------------------
+string StartWinThread::get_server_name_with_cotes(string servname)
+{
+	string::size_type	pos = servname.find_last_of("/\\");
+	if (pos!=string::npos)
+	{
+		/*
+		string	str("\"");
+		str += servname.substr(0, pos);
+		str += "\"";
+		str += servname.substr(pos);
+		
+		string	str("\"");
+		str += servname;
+		str += "\"";
+		return str;
+		*/
+		string	str("c:\\\"Program Files\"\\Tango\\notify_daemon.bat");
+		return str;
+	}
+	else
+		return servname;
+}
 //+----------------------------------------------------------
 //	WIN 32 Thread to fork a sub process
 //		If batch file, the spawnv is blocking !!!
@@ -320,16 +391,27 @@ void *StartWinThread::run_undetached(void *ptr)
 {
 	//	Check if batch file
 	string	str_server(process->servname);
+	string	str_with_cotes = get_server_name_with_cotes(process->servname);
+	char	*servname = new char[str_with_cotes.length()+1];
+	strcpy(servname, str_with_cotes.c_str());
 	
+	string cmd(servname);
+	cmd += "  ";
+	cmd += process->instancename;
+	cout << "system(" << cmd << ");" << endl;
+	system(cmd.c_str());
+	return NULL;
+
 	if (str_server.find(".bat") != string::npos)
 	{
 		char	*argv[3];
 		int		i = 0;
-		argv[i++] = process->servname;
+
+		argv[i++] = servname;
 		argv[i++] = process->instancename;
 		argv[i++] = NULL;
 
-		cout << "Forking "   << process->servname << endl;
+		cout << "Forking "   << servname << endl;
 		cout << "log file: " << process->logfile << endl;
 
 		//	Write a starting message in log file
@@ -348,8 +430,10 @@ void *StartWinThread::run_undetached(void *ptr)
 			ofs1 << "Exec(" << argv[0] << ") failed " << endl;
 			ofs1 << "_spawnv has returned " << ret << endl;
 			DWORD	errcode = GetLastError();
-		    LPTSTR lp_err;
-			if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ARGUMENT_ARRAY,
+		    LPTSTR	lp_err;
+			if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
+								FORMAT_MESSAGE_FROM_SYSTEM     |
+								FORMAT_MESSAGE_ARGUMENT_ARRAY,
 							NULL,
 							errcode,
 							LANG_NEUTRAL,
@@ -363,6 +447,10 @@ void *StartWinThread::run_undetached(void *ptr)
 					*p++ = lp_err[i];
 				*p = '\0';
 				ofs1 << str << " errcode = " << errcode << endl;
+				cout << "===============================================" << endl
+					 << servname << "  " << process->instancename<< endl
+					 << str << " errcode = " << errcode                   << endl
+					 << "===============================================" << endl;
 				::LocalFree(lp_err);
 			}
 			ofs1.close();
@@ -378,7 +466,7 @@ void *StartWinThread::run_undetached(void *ptr)
 		PROCESS_INFORMATION pi;
 		STARTUPINFO si;
 
-		string	cmd_line(process->servname);
+		string	cmd_line(servname);
 		cmd_line += " ";
 		cmd_line += process->instancename;
 
@@ -462,10 +550,11 @@ void *StartWinThread::run_undetached(void *ptr)
 		delete lpw_logfile;
 		ofs.close();
 	}
+	delete[] servname;
 	return NULL;
 }
 
-#endif	//	WIN32
+#endif	//	_TG_WINDOWS_
 
 //+------------------------------------------------------------------
 //+------------------------------------------------------------------
