@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.42 2009/04/06 13:57:56 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.46 2010/10/15 06:20:33 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -13,9 +13,42 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.42 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.46 $
 //
 // $Log: Starter.cpp,v $
+// Revision 3.46  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.45  2010/10/08 08:48:50  pascal_verdier
+// Include files order changed.
+//
+// Revision 3.44  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.43  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.42  2009/04/06 13:57:56  pascal_verdier
 // IDL-4 (Tango-7) release.
 //
@@ -191,17 +224,11 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 // Revision 1.1  2000/07/31 13:48:42  verdier
 // Initial revision
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
 //-=============================================================================
 //
 //  		This file is generated by POGO
 //	(Program Obviously used to Generate tango Object)
 //
-//             (c) - Pascal Verdier - ESRF
 //=============================================================================
 
 
@@ -228,7 +255,9 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //===================================================================
 
 
-#ifdef WIN32
+#include <tango.h>
+
+#ifdef _TG_WINDOWS_
 #	include <process.h>
 #	include <direct.h>
 #	include <io.h>
@@ -244,7 +273,6 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 #include <fcntl.h>
 #include <signal.h>
 
-#include <tango.h>
 #include <StarterUtil.h>
 #include <Starter.h>
 #include <StarterClass.h>
@@ -252,7 +280,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 
 
 #ifndef	TIME_VAR
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 
 #	define	TimeVal	struct timeval
 #	define	GetTime(t)	gettimeofday(&t, NULL);
@@ -267,7 +295,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 #	define	Elapsed(before, after)	\
 		1000*(after.time - before.time) + (after.millitm - before.millitm)
 
-#endif	/*	WIN32		*/
+#endif	/*	_TG_WINDOWS_		*/
 #endif	/*	TIME_VAR	*/
 
 //#define DisplayTime
@@ -359,7 +387,6 @@ void Starter::init_device()
 	//	First time, check if instance and host name are coherent
 	check_host();
 
-
 	//	Do it only at startup and not at Init command
 	//----------------------------------------------------
 	if (starting==true)
@@ -406,7 +433,7 @@ void Starter::init_device()
 		}
 		//	Create a StarterUtil instance
 		//--------------------------------------
-		util = new StarterUtil(dbase, hosts_list);
+		util = new StarterUtil(dbase, hosts_list, logFileHome);
 		util->log_starter_info("Starter startup");
 
 		//	Initialize Attribute data member
@@ -456,7 +483,7 @@ void Starter::init_device()
 			catch(Tango::DevFailed &e) {
 				Tango::Except::print_exception(e);
 			}
-#			ifdef WIN32
+#			ifdef _TG_WINDOWS_
 				_sleep(1000);
 #			else
 				sleep(1);
@@ -464,7 +491,7 @@ void Starter::init_device()
 		}
 		
 //	A a wait for first ping timeout !!!!
-#	ifdef WIN32
+#	ifdef _TG_WINDOWS_
 		_sleep(3000);
 #	else
 		sleep(3);
@@ -536,6 +563,7 @@ void Starter::get_device_property()
 	dev_prop.push_back(Tango::DbDatum("InterStartupLevelWait"));
 	dev_prop.push_back(Tango::DbDatum("ServerStartupTimeout"));
 	dev_prop.push_back(Tango::DbDatum("KeepLogFiles"));
+	dev_prop.push_back(Tango::DbDatum("LogFileHome"));
 
 	//	Call database and extract values
 	//--------------------------------------------
@@ -623,6 +651,17 @@ void Starter::get_device_property()
 	//	And try to extract KeepLogFiles value from database
 	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  keepLogFiles;
 
+	//	Try to initialize LogFileHome from class property
+	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
+	if (cl_prop.is_empty()==false)	cl_prop  >>  logFileHome;
+	else {
+		//	Try to initialize LogFileHome from default device value
+		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
+		if (def_prop.is_empty()==false)	def_prop  >>  logFileHome;
+	}
+	//	And try to extract LogFileHome value from database
+	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  logFileHome;
+
 
 
 	//	End of Automatic code generation
@@ -652,7 +691,8 @@ void Starter::get_device_property()
 	transform(tmp.begin(), tmp.end(), tmp.begin(), ::tolower);
 	if (tmp=="false")
 		fireFromDbase = false;
-	cout << "fireFromDbase = " << fireFromDbase << endl;
+	cout << "fireFromDbase  = " << fireFromDbase << endl;
+	cout << "logFileHome    = " << logFileHome   << endl;
 }
 //+----------------------------------------------------------------------------
 //
@@ -950,7 +990,7 @@ NewProcess *Starter::processCouldStart(char *argin)
 	string	log_file = util->build_log_file_name(argin);
 	
 	string	logpath;
-	LogPath(logpath);
+	LogPath(logpath,logFileHome);
 	INFO_STREAM << "LOG file : " << log_file << endl;
 	if (chdir(logpath.c_str())==-1)
 	{
@@ -960,7 +1000,7 @@ NewProcess *Starter::processCouldStart(char *argin)
 			//-------------------------
 			cerr << "ENOENT" << endl;
 			cerr << errno << "  " << strerror(errno) << endl;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			mkdir(TmpRoot);
 			int r = mkdir(logpath.c_str());
 #else
@@ -1475,7 +1515,7 @@ void Starter::hard_kill_server(Tango::DevString argin)
 					tms.str().c_str(),
 					(const char *)"Starter::hard_kill_server()");
 	}
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 
 	HANDLE	handle = NULL;				//- process addr (in the heap)
 	if( (handle=OpenProcess(PROCESS_TERMINATE, false, pid)) == NULL)
@@ -1552,8 +1592,4 @@ void Starter::check_host()
 	}
 }
 
-
-
-
-
 }	//	namespace
