@@ -8,9 +8,39 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.25 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.28 $
 //
 // $Log: Starter.h,v $
+// Revision 3.28  2010/10/15 06:20:32  pascal_verdier
+// Copyright added.
+//
+// Revision 3.27  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.26  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.25  2009/04/06 13:57:57  pascal_verdier
 // IDL-4 (Tango-7) release.
 //
@@ -129,17 +159,11 @@
 // Revision 1.1  2000/07/31 13:48:42  verdier
 // Initial revision
 //
-//
-// copyleft :    European Synchrotron Radiation Facility
-//               BP 220, Grenoble 38043
-//               FRANCE
-//
 //=============================================================================
 //
 //  		This file is generated by POGO
 //	(Program Obviously used to Generate tango Object)
 //
-//             (c) - Pascal Verdier - ESRF
 //=============================================================================
 #ifndef _STARTER_H
 #define _STARTER_H
@@ -150,7 +174,7 @@
 
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.25 $
+ * @version	$Revision: 3.28 $
  */
 
  //	Add your own constants definitions here.
@@ -164,6 +188,13 @@
 #define NB_SERVERS_MAX			200
 #define SERVER_TIMEOUT		30	//	Timeout Minimum at server startup
 
+
+#ifdef _TG_WINDOWS_
+#	define	LOG_HOME	"c:\\temp"
+#else
+#	define	LOG_HOME	"/var/tmp"
+#endif
+
 //	Structure to difine a new process to start
 typedef struct {
 	bool	could_start;
@@ -294,6 +325,12 @@ public :
  *	Number of log file kept.
  */
 	Tango::DevLong	keepLogFiles;
+/**
+ *	The home directory to log servers traces.
+ *	For Linux the default value is /var/tmp
+ *	For Win32 it is c:\temp
+ */
+	string	logFileHome;
 //@}
 
 /**@name Constructors
@@ -584,7 +621,7 @@ public:
 };
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 class StartWinThread: public omni_thread
 {
 	NewProcess	*process;
@@ -593,6 +630,9 @@ public:
 	StartWinThread(NewProcess* proc, Starter *st) 
 		{ process = proc; starter=st;};
 
+	// Set the path between cotes for windows.
+	string get_server_name_with_cotes(string servname);
+	
 	void *run_undetached(void *);
 	void start() {start_undetached();}
 };
