@@ -1,7 +1,7 @@
-static const char *ClassId    = "$Id: StarterClass.cpp,v 3.27 2009/04/06 13:57:57 pascal_verdier Exp $";
+static const char *ClassId    = "$Id: StarterClass.cpp,v 3.31 2010/10/15 06:20:33 pascal_verdier Exp $";
 static const char *CvsPath    = "$Source: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v $";
 static const char *SvnPath    = "$HeadURL: $";
-static const char *RcsId     = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.27 2009/04/06 13:57:57 pascal_verdier Exp $";
+static const char *RcsId     = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.31 2010/10/15 06:20:33 pascal_verdier Exp $";
 static const char *TagName   = "$Name:  $";
 static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
 //+=============================================================================
@@ -17,9 +17,42 @@ static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.27 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.31 $
 //
 // $Log: StarterClass.cpp,v $
+// Revision 3.31  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.30  2010/10/08 08:48:50  pascal_verdier
+// Include files order changed.
+//
+// Revision 3.29  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.28  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.27  2009/04/06 13:57:57  pascal_verdier
 // IDL-4 (Tango-7) release.
 //
@@ -145,17 +178,11 @@ static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/
 // Revision 1.1  2000/07/31 13:48:42  verdier
 // Initial revision
 //
-//
-// copyleft :   European Synchrotron Radiation Facility
-//              BP 220, Grenoble 38043
-//              FRANCE
-//
 //-=============================================================================
 //
 //  		This file is generated by POGO
 //	(Program Obviously used to Generate tango Object)
 //
-//             (c) - Pascal Verdier - ESRF
 //=============================================================================
 
 
@@ -693,7 +720,8 @@ void StarterClass::get_class_property()
 	readInfoDbPeriod  = 4;
 	nbStartupLevels   = 5;
 	cmdPollingTimeout = 60;
-	useEvents = false;
+	useEvents   = false;
+	logFileHome = LOG_HOME;
 
 	//	Read class properties from database.(Automatic code generation)
 	//-------------------------------------------------------------
@@ -702,6 +730,7 @@ void StarterClass::get_class_property()
 	cl_prop.push_back(Tango::DbDatum("CmdPollingTimeout"));
 	cl_prop.push_back(Tango::DbDatum("UseEvents"));
 	cl_prop.push_back(Tango::DbDatum("ServerStartupTimeout"));
+	cl_prop.push_back(Tango::DbDatum("LogFileHome"));
 
 	//	Call database and extract values
 	//--------------------------------------------
@@ -775,12 +804,31 @@ void StarterClass::get_class_property()
 		}
 	}
 
+	//	Try to extract LogFileHome value
+	if (cl_prop[++i].is_empty()==false)	cl_prop[i]  >>  logFileHome;
+	else
+	{
+		//	Check default value for LogFileHome
+		def_prop = get_default_class_property(cl_prop[i].name);
+		if (def_prop.is_empty()==false)
+		{
+			def_prop  >>  logFileHome;
+			cl_prop[i]  <<  logFileHome;
+		}
+	}
+
 	//	End of Automatic code generation
 	//-------------------------------------------------------------
 	cout2 << "readInfoDbPeriod  = " << readInfoDbPeriod << endl;
 	cout2 << "nbStartupLevels   = " << nbStartupLevels << endl;
 	cout2 << "cmdPollingTimeout = " << cmdPollingTimeout << endl;
 	cout2 << "useEvents         = " << ((useEvents)? "True":"False") << endl;
+	cout2 << "logFileHome    = " << logFileHome   << endl;
+
+	//	Put the value (depends on OS) in cl_prop.
+	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
+		if (cl_prop[i].name == "LogFileHome")
+			cl_prop[i]  <<  logFileHome;
 }
 
 //+----------------------------------------------------------------------------
@@ -874,6 +922,20 @@ void StarterClass::set_default_property()
 	else
 		add_wiz_class_prop(prop_name, prop_desc);
 
+	prop_name = "LogFileHome";
+	prop_desc = "The home directory to log servers traces.\nFor Linux the default value is /var/tmp\nFor Win32 it is c:\temp";
+	prop_def  = "";
+	vect_data.clear();
+	if (prop_def.length()>0)
+	{
+		Tango::DbDatum	data(prop_name);
+		data << vect_data ;
+		cl_def_prop.push_back(data);
+		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
+	}
+	else
+		add_wiz_class_prop(prop_name, prop_desc);
+
 	//	Set Default Device Properties
 	prop_name = "StartDsPath";
 	prop_desc = "Path to find executable files\nto start device servers";
@@ -979,6 +1041,20 @@ void StarterClass::set_default_property()
 	else
 		add_wiz_dev_prop(prop_name, prop_desc);
 
+	prop_name = "LogFileHome";
+	prop_desc = "The home directory to log servers traces.\nFor Linux the default value is /var/tmp\nFor Win32 it is c:\temp";
+	prop_def  = "";
+	vect_data.clear();
+	if (prop_def.length()>0)
+	{
+		Tango::DbDatum	data(prop_name);
+		data << vect_data ;
+		dev_def_prop.push_back(data);
+		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
+	}
+	else
+		add_wiz_dev_prop(prop_name, prop_desc);
+
 }
 //+----------------------------------------------------------------------------
 //
