@@ -12,9 +12,39 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.12 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.15 $
 //
 // $Log: StarterClass.h,v $
+// Revision 3.15  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.14  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.13  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.12  2008/04/07 13:10:33  pascal_verdier
 // New pogo property management.
 //
@@ -86,17 +116,11 @@
 // Revision 1.3  2001/02/12 09:34:21  verdier
 // SunOS and Linux OK.
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
 //=============================================================================
 //
 //  		This file is generated by POGO
 //	(Program Obviously used to Generate tango Object)
 //
-//             (c) - Pascal Verdier - ESRF
 //=============================================================================
 
 #ifndef _STARTERCLASS_H
@@ -419,7 +443,7 @@ public:
 //
 
 class
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	__declspec(dllexport)
 #endif
 	StarterClass : public Tango::DeviceClass
@@ -446,6 +470,12 @@ public:
  *	Timeout on device server startup in seconds.
  */
 	Tango::DevLong	serverStartupTimeout;
+/**
+ *	The home directory to log servers traces.
+For Linux the default value is /var/tmp
+For Win32 it is c:\temp
+ */
+	string	logFileHome;
 
 //	add your own data members here
 //------------------------------------
