@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.23 2008/12/12 13:29:56 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.27 2010/10/15 06:20:33 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         StarterUtil.cpp
@@ -9,9 +9,42 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.23 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.27 $
 //
 // $Log: StarterUtil.cpp,v $
+// Revision 3.27  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.26  2010/10/08 08:48:50  pascal_verdier
+// Include files order changed.
+//
+// Revision 3.25  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.24  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.23  2008/12/12 13:29:56  pascal_verdier
 // Log in file start and stop for servers and itself.
 //
@@ -121,26 +154,21 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 // Revision 1.1  2001/02/12 09:34:21  verdier
 // Initial revision
 //
-//
-// copyleft :     European Synchrotron Radiation Facility
-//                BP 220, Grenoble 38043
-//                FRANCE
-//
 //-=============================================================================
 
-#include <stdio.h>
+#include <tango.h>
 
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 #	include <sys/time.h>
 #endif
 
 
-#include <tango.h>
 #include <StarterUtil.h>
 #include <sstream>
 
@@ -149,24 +177,23 @@ namespace Starter_ns
 {
 
 int StarterUtil::elapsed;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 struct timeval	StarterUtil::before, StarterUtil::after;
 #else
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 //+------------------------------------------------------------------
 /**
  *	Contructor - Initialize data members.
  */
 //+------------------------------------------------------------------
-StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names)
+StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names, string logHome)
 {
 	dbase = database;
-	//hostnames     = host_names;
 	notifyd_name  = "notifd/factory/";
-//	notifyd_name  = "dserver/starter/";
 	notifyd_name += host_names[0];
 	ch_factory   = NULL;
+	log_home     = logHome;
 	
 	//	Remove the Fully Qualify Domain Name for tango less than 5.2 compatibility
 	for (unsigned int i=0 ; i<host_names.size() ; i++)
@@ -176,15 +203,17 @@ StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names
 	proc_util->start();
 	//	Wait a bit to be sure 
 	//	that runningp rocess list is updated.
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	_sleep(1000);
 #else
 	sleep(1);
 #endif
 
 	//	Build starter log file name.
-	LogPath(starter_log_file);
+	LogPath(starter_log_file, logHome);
 	starter_log_file += "/Starter.log";
+
+	cout << "---->  starter_log_file = " << starter_log_file << endl;
 }
 //+------------------------------------------------------------------
 /**
@@ -253,7 +282,7 @@ char *StarterUtil::check_exe_file(string name)
 	if (result==NULL)
 		free(result);
 	string	filename(name);
-#ifdef	WIN32
+#ifdef	_TG_WINDOWS_
 	filename += ".exe";
 #endif
 	//cout << "Checking " << filename << endl;
@@ -265,7 +294,7 @@ char *StarterUtil::check_exe_file(string name)
 		strcpy(result, filename.c_str());
 		return result;
 	}
-#ifdef	WIN32
+#ifdef	_TG_WINDOWS_
 
 	//	Check for catch file
 	filename = name;
@@ -467,7 +496,7 @@ vector<string> StarterUtil::get_log_file_list(string logfile)
 		string	filter = str.substr(0, pos);
 		filter += "_[";
 		
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		cout << "Searching " << filter << "  in " << path << endl;
 		DIR		*dir = opendir ((char *)path.c_str()) ;
 		if(dir==NULL)
@@ -571,7 +600,7 @@ string StarterUtil::build_log_file_name(char *server)
 	//	And create full name with path
 	//-----------------------------------------
 	string	log_file;
-	LogPath(log_file);
+	LogPath(log_file,log_home);
 	log_file += slash;
 	log_file += servname;
 	log_file += "_";
@@ -872,7 +901,6 @@ Tango::DevState StarterUtil::is_notifyd_alive()
 		//cout << notify_procname << " is NOT running" << endl;
  		notifd_state = Tango::FAULT;
 	}
-
 	return notifd_state;
 }
 
