@@ -8,9 +8,39 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.13 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.16 $
 //
 // $Log: StarterUtil.h,v $
+// Revision 3.16  2010/10/15 06:20:33  pascal_verdier
+// Copyright added.
+//
+// Revision 3.15  2010/09/21 12:18:58  pascal_verdier
+// GPL Licence added to header.
+//
+// Revision 3.14  2010/02/09 15:09:49  pascal_verdier
+// Define  _TG_WINDOWS_  replace WIN32.
+// LogFileHome property added.
+//
 // Revision 3.13  2008/12/12 13:29:56  pascal_verdier
 // Log in file start and stop for servers and itself.
 //
@@ -33,12 +63,8 @@
 // Revision 3.7  2006/02/09 11:59:19  pascal_verdier
 // A ping thread is now started for each server.
 //
-//
-// copyleft :    European Synchrotron Radiation Facility
-//               BP 220, Grenoble 38043
-//               FRANCE
-//
 //=============================================================================
+
 #ifndef _STARTER_UTIL_H
 #define _STARTER_UTIL_H
 
@@ -48,7 +74,7 @@
 
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.13 $
+ * @version	$Revision: 3.16 $
  */
 
 /**
@@ -61,7 +87,7 @@ namespace Starter_ns
 
  //	Add your own constants definitions here.
  //-----------------------------------------------
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 typedef unsigned char boolean;
 #endif
 
@@ -79,7 +105,7 @@ ControledServer;
 
 //	Millisecond sleep platform independant.
 //--------------------------------------------
-#	ifdef WIN32
+#	ifdef _TG_WINDOWS_
 #	define		ms_sleep(ms)	_sleep(ms);
 #	else
 #	define		ms_sleep(ms)	{\
@@ -92,15 +118,15 @@ ControledServer;
 
 //	Definitions for separators
 //----------------------------------
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #	define	slash	'\\'
 #	define	TmpRoot	"c:\\temp"
 #else
 #	define	slash	'/'
 #	define	TmpRoot	"/var/tmp"
 #endif
-#define	LogPath(s)		\
-		s = TmpRoot;	\
+#define	LogPath(s,home)		\
+		s  = home;	\
 		s += slash;		\
 		s += "ds.log";
 
@@ -113,6 +139,7 @@ class StarterUtil
 {
 public :
 	string				notifyd_name;
+	string				log_home;
 	string				starter_log_file;
 	CheckProcessUtil	*proc_util;
 
@@ -128,7 +155,7 @@ public :
 /**
  *	Default constructor.
  */
-StarterUtil(Tango::DeviceProxy *dev, vector<string> host_name);
+StarterUtil(Tango::DeviceProxy *dev, vector<string> host_name, string logHome);
 //@}
 /**
  *	@name methods
@@ -230,10 +257,10 @@ private:
 	 */
 	Tango::DeviceProxy	*dbase;
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	static struct timeval	before, after;
 #else
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 };
 
 
