@@ -7,7 +7,7 @@
 //
 // $Author: taurel $
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -27,9 +27,15 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.11 $
+// $Revision: 3.13 $
 //
 // $Log: accessproxy.cpp,v $
+// Revision 3.13  2010/09/09 13:43:38  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.12  2010/02/08 14:40:19  taurel
+// - Add a patch from Nicolas about Mac-OS port
+//
 // Revision 3.11  2009/09/22 11:04:45  taurel
 // - Environment variables in file also supported for Windows
 //
@@ -234,15 +240,13 @@ AccessControlType AccessProxy::check_access_control(string &devname)
 #endif
 				struct hostent *my_addr_ptr;
 
-#ifdef sun
+#if defined (sun)
 				if (gethostbyname_r(h_name,&my_addr,buffer,sizeof(buffer),&err) != 0)
-#else
-#ifdef _TG_WINDOWS_
+#elif defined (_TG_WINDOWS_) || defined (__MACOS__) || defined (__darwin__)
 				my_addr_ptr = gethostbyname(h_name);
 				if (my_addr_ptr == NULL)
 #else
 				if (gethostbyname_r(h_name,&my_addr,buffer,sizeof(buffer),&my_addr_ptr,&err) != 0)
-#endif
 #endif
 				{
 					cerr << "AccessProxy::check_access_control: Can't get my IP address !" << endl;
