@@ -1,11 +1,11 @@
-static const char *RcsId = "$Id: api_util.cpp,v 3.28 2009/08/27 07:22:43 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: api_util.cpp,v 3.34 2010/09/12 12:18:23 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //
 // cpp 	- C++ source code file for TANGO api class ApiUtil
 //
 // programmer 	- Emmanuel Taurel (taurel@esrf.fr)
 //
-// Copyright (C) :      2002,2003,2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2002,2003,2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -28,6 +28,29 @@ static const char *RcsId = "$Id: api_util.cpp,v 3.28 2009/08/27 07:22:43 taurel
 // original 	- May 2002
 //
 // $Log: api_util.cpp,v $
+// Revision 3.34  2010/09/12 12:18:23  taurel
+// - Now, the test suite seems OK
+//
+// Revision 3.33  2010/09/09 13:43:38  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.32  2010/09/09 13:28:03  taurel
+// - Commit after the last merge with the bugfixes branch
+// - Fix some warning when compiled -W -Wall
+//
+// Revision 3.31  2010/09/08 12:32:11  taurel
+// - Miscellaneous changes to implement a better timeout management
+// (now manage a user connect timeout with the env. variable TANGOconnectTimeout)
+//
+// Revision 3.30  2010/08/25 11:37:12  taurel
+// - Just some beautifulling!!
+//
+// Revision 3.29  2010/01/07 08:35:06  taurel
+// - Several change sto improve thread safety of the DeviceProxy, AttributeProxy, ApiUtul and EventConsumer classes
+// Revision 3.28.2.1  2010/06/23 14:10:23  taurel
+// - Full Tango as described in doc Appendix C is now also supported
+// for group
+//
 // Revision 3.28  2009/08/27 07:22:43  taurel
 // - Commit after anothre merge with Release_7_0_2-bugfixes branch
 //
@@ -293,15 +316,6 @@ namespace Tango
 ApiUtil *ApiUtil::_instance = NULL;
 omni_mutex ApiUtil::inst_mutex;
 
-/*ApiUtil *ApiUtil::instance()
-{
-	omni_mutex_lock lo(inst_mutex);
-	
-	if (_instance == NULL)
-		_instance = new ApiUtil();
-	return _instance;
-}*/
-
 //+----------------------------------------------------------------------------
 //
 // method : 		ApiUtil::ApiUtil()
@@ -357,6 +371,20 @@ ApiUtil::ApiUtil():exit_lock_installed(false),reset_already_executed_flag(false)
 #else
 	ext->cl_pid = getpid();
 #endif
+
+//
+// Check if the user has defined his own connection timeout
+//
+
+	string var;
+	if (get_env_var("TANGOconnectTimeout",var) == 0)
+	{
+		int user_to = -1;
+		istringstream iss(var);
+		iss >> user_to;
+		if (iss)
+			ext->user_connect_timeout = user_to;
+	}
 }
 
 //+----------------------------------------------------------------------------
@@ -424,11 +452,10 @@ ApiUtil::~ApiUtil()
 			{
 				_orb->destroy();
 			}
-			catch (...) {}
+			catch (...) {cout << "In the exception handler"<<endl;}
 		}
 		CORBA::release(_orb);
 	}
-	
 
 }
 
@@ -882,6 +909,11 @@ void ApiUtil::create_event_consumer()
 	ext->event_consumer = EventConsumer::create();
 }
 
+EventConsumer *ApiUtil::get_event_consumer()
+{	
+	return ext->event_consumer;
+}
+	
 //+----------------------------------------------------------------------------
 //
 // method : 		ApiUtil::clean_locking_threads()
@@ -1620,6 +1652,27 @@ void ApiUtil::device_to_attr(const DeviceAttribute &dev_attr,AttributeValue &att
 	}
 }
 
+//-----------------------------------------------------------------------------
+//
+// method : 		ApiUtil::get_env_var()
+// 
+// description : 	
+//
+// argin(s) :		- env_var_name : The environment variable name
+//					- env_var : Reference to the string initialised with
+//								the env. variable value (if found)
+//
+// This method returns 0 if the env. variable is found and -1 otherwise
+//
+//-----------------------------------------------------------------------------
+
+int ApiUtil::get_env_var(const char *env_var_name,string &env_var)
+{
+	DummyDeviceProxy d;
+	return d.get_env_var(env_var_name,env_var);
+}
+
+
 //+-------------------------------------------------------------------------
 //
 // operator overloading : 	<<
