@@ -2,7 +2,7 @@
 // dbapi.h -	include file for TANGO database api
 //
 // 
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -74,7 +74,8 @@ class DatabaseExt
 public:
 	DatabaseExt():db_tg(NULL) {};
 	
-	Tango::Util *db_tg;
+	Tango::Util 	*db_tg;
+	omni_mutex		map_mutex;
 };
 
 class Database : public Tango::Connection
@@ -83,13 +84,8 @@ private :
 	virtual string get_corba_name(bool);
 	virtual string build_corba_name() {return string("nada");}
 	virtual int get_lock_ctr() {return 0;}
-	virtual void set_lock_ctr(int lo) {}
-	
-	inline string dev_name() 
-	{if (db_device_name.empty() == true)
-		{CORBA::String_var n = device->name();db_device_name = n;}
-	return db_device_name; }
-	
+	virtual void set_lock_ctr(int) {}
+		
 	bool				db_multi_svc;
 	vector<string>		multi_db_port;
 	vector<string>		multi_db_host;
@@ -111,7 +107,10 @@ private :
 	vector<DbHistory> make_history_array(bool, CORBA::Any_var &);
 	
 	void check_access();
-
+	inline string dev_name();
+	void get_server_release();
+	void check_access_and_get();
+	
 public :
 	Database(CORBA::ORB *orb=NULL);
 	Database(string &host, int port, CORBA::ORB *orb=NULL);
@@ -119,7 +118,9 @@ public :
 	
 	void write_filedatabase();
 	void reread_filedatabase();
+	void write_event_channel_ior_filedatabase(string &,string &);
 	void build_connection ();
+	void post_reconnection();
 	~Database();
 	inline Device_var &get_dbase() { return device;}
 	void check_tango_host(const char *);
@@ -137,6 +138,8 @@ public :
 	vector<string> &get_multi_host() {return multi_db_host;}
 	vector<string> &get_multi_port() {return multi_db_port;}
 
+	const string &get_file_name();
+
 #ifdef _TG_WINDOWS_
 	Database(CORBA::ORB *orb,string &,string &);
 	long get_tango_host_from_reg(char **,string &,string &);
@@ -257,6 +260,22 @@ public :
 
 };
 
+//
+// Some Database class inline methods
+//
+
+
+inline string Database::dev_name() 
+{
+	if (db_device_name.empty() == true)
+	{
+		CORBA::String_var n = device->name();
+		db_device_name = n;
+	}
+	return db_device_name;
+}
+	
+	
 //
 // Some macros to call the Db server
 // These macros will do some retries in case of
@@ -370,6 +389,7 @@ public :
 	~DbDevice();
 	void set_name(string &new_name) {name = new_name;}
 	Database *get_dbase();
+	void set_dbase(Database *db) {dbase = db;}
 //
 // methods
 //
@@ -715,7 +735,7 @@ public:
 	const PropEltIdx &get_Default_prop() {return Default_prop;}
 	const PropEltIdx &get_adm_dev_prop() {return adm_dev_prop;}
 	const PropEltIdx &get_ctrl_serv_prop() {return ctrl_serv_prop;}
-	const int get_class_nb() {return class_nb;}
+	int get_class_nb() {return class_nb;}
 	const ClassEltIdx *get_classes_elt() {return classes_idx;}
 	int get_data_nb() {return n_data;}
 	
