@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_device.cpp,v 3.9 2009/04/30 09:22:23 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: dbapi_device.cpp,v 3.11 2010/09/09 13:43:38 taurel Exp $\n$Name: Release_7_2_0 $";
 //
 // dbdevice.cpp - C++ source code file for TANGO dbapi class DbDevice
 //
@@ -6,7 +6,7 @@ static const char *RcsId = "$Id: dbapi_device.cpp,v 3.9 2009/04/30 09:22:23 taur
 //
 // original 	- October 2000
 //
-// Copyright (C) :      2000,2001,2002,2003,2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -290,7 +290,9 @@ AccessControlType DbDevice::check_access_control()
 	else
 	{
 		ApiUtil *au = ApiUtil::instance();
-		AccessControlType acc = (au->get_db_vect()[db_ind])->check_access_control(name);
+		Database *db = au->get_db_vect()[db_ind];
+		AccessControlType acc = db->check_access_control(name);
+		db->set_access_checked(true);
 		return acc;
 	}
 }
