@@ -1,4 +1,4 @@
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -322,7 +322,7 @@ int FileDatabase :: class_lex(string& tmp_word)
 
 
 // ****************************************************
-// Go to the next line                                      */
+// Go to the next line                                */
 // ****************************************************
 void  FileDatabase :: jump_line(ifstream& f) 
 {
@@ -333,7 +333,7 @@ void  FileDatabase :: jump_line(ifstream& f)
 
 void  FileDatabase :: jump_space(ifstream& f)
 {
-	while((CurrentChar<=32) && (CurrentChar>0)) 
+	while((CurrentChar<=32) && (CurrentChar>0))
 		read_char(f);
 }
 
@@ -452,12 +452,12 @@ string FileDatabase:: read_full_word(ifstream& f)
   		read_char(f);
   		while( CurrentChar!='"' && CurrentChar!=0 && CurrentChar!='\n')
 		{
-    			ret_word += CurrentChar;
-    			read_char(f);
+			ret_word += CurrentChar;
+			read_char(f);
   		}
   		if( CurrentChar==0 || CurrentChar=='\n')
   		{
-    			cout3 << "Warning: String too long at line " << StartLine << endl;
+			cout3 << "Warning: String too long at line " << StartLine << endl;
 			TangoSys_MemStream desc;
 			desc << "File database: String too long at line " << StartLine;
 			desc << " in file " << filename << "." << ends;
@@ -466,6 +466,8 @@ string FileDatabase:: read_full_word(ifstream& f)
 				       (const char *)"FileDatabase::read_full_word");
   		}
   		read_char(f);
+		if (ret_word.length() == 0)
+			ret_word = string(lexical_word_null);
   		return ret_word;
 	}
 
@@ -490,7 +492,7 @@ void FileDatabase:: CHECK_LEX(int lt,int le)
 {
    	if( lt!=le )
 	{
-    		cout3 << "Error at line " << StartLine << endl;
+		cout3 << "Error at line " << StartLine << endl;
 		TangoSys_MemStream desc;
 		desc << "File database: Error in file at line " << StartLine;
 		desc << " in file " << filename << "." << ends;
@@ -517,24 +519,22 @@ vector<string> FileDatabase:: parse_resource_value(ifstream& f)
 		lex=class_lex(word);
  
   	/* allow ... ,\ syntax */
-    		if( lex==_TG_ASLASH )
+		if( lex==_TG_ASLASH )
 		{
-      			word=read_full_word(f);
+			word=read_full_word(f);
 			lex=class_lex(word);
-    		}
+		}
      
-    		CHECK_LEX(lex,_TG_STRING);
+		CHECK_LEX(lex,_TG_STRING);
 
-    		ret.push_back(word);
-    		nbr++;
-     
-    		word=read_word(f);
-		lex=class_lex(word);
+		ret.push_back(word);
+		nbr++;
      
+		word=read_word(f);
+		lex=class_lex(word);   
   	}
-   
+
 	return ret;
-   
 }
 
   // ****************************************************
@@ -543,11 +543,11 @@ vector<string> FileDatabase:: parse_resource_value(ifstream& f)
   // ****************************************************
 
 
-string FileDatabase::parse_res_file(const string& file_name)
+std::string FileDatabase::parse_res_file(const std::string &file_name)
 {
-    	ifstream f;
-    	bool eof=false;
-    	int lex;
+	ifstream f;
+	bool eof=false;
+	int lex;
 
 	t_tango_class* un_class;
 	t_device* un_device;
@@ -556,41 +556,42 @@ string FileDatabase::parse_res_file(const string& file_name)
 	string family;
 	string member;
 	string name;
-    	string prop_name;
+	string prop_name;
 
-    	CrtLine=1;
-    	NextChar=' ';
-    	CurrentChar=' ';
+	CrtLine=1;
+	NextChar=' ';
+	CurrentChar=' ';
 
-    	cout4 << "FILEDATABASE: entering parse_res_file" << endl;
-     /* OPEN THE FILE                  */
+	cout4 << "FILEDATABASE: entering parse_res_file" << endl;
 
-    	f.open (file_name.c_str(), ifstream::in);
-    	if ( !f.good() )
-    	{
+/* OPEN THE FILE                  */
+
+	f.open (file_name.c_str(), ifstream::in);
+	if ( !f.good() )
+	{
 		TangoSys_MemStream desc;
 		desc << "FILEDATABASE could not open file " << file_name << "." << ends;
 		ApiConnExcept::throw_exception((const char *)"API_DatabaseFileError",
 				       desc.str(),
 				       (const char *)"FileDatabase::parse_res_file");
-    	}
+	}
 
 /* CHECK BEGINING OF CONFIG FILE  */
 
-     	word=read_word(f);
-     	if( word == "" ) 
+	word=read_word(f);
+	if( word == "" ) 
 	{
 		f.close();
-        	return file_name + " is empty...";
+		return file_name + " is empty...";
 	}
-     	lex=class_lex(word);
+	lex=class_lex(word);
 	m_server.name = word;
 	
 /* PARSE                          */
 
-     	while( !eof )
-     	{
-       		switch(lex)
+	while( !eof )
+	{
+		switch(lex)
 		{
 /* Start a resource mame */
          	case _TG_STRING:
@@ -982,14 +983,21 @@ void  FileDatabase :: write_file()
 			int margin = 6 + (*it)->name.size() + 2 + (*itp)->name.size() + 2;
 			string margin_s(margin,' ');
 			vector<string>::iterator iterator_s = (*itp)->value.begin();
+			if ((*iterator_s).length() == 0)
+				(*iterator_s)[0] = ' ';
 			if (iterator_s != (*itp)->value.end())
 			{
 				//f << "\"" << (*iterator_s) << "\"";
-				if (iterator_s->find(' ', 0)!=string::npos) 
+				if ((*iterator_s).length() == 0)
+					f << "\"\"";
+				else
+				{
+					if (iterator_s->find(' ', 0)!=string::npos)
 						f << "\"" ;
 					f << (*iterator_s);
-				if (iterator_s->find(' ', 0)!=string::npos) 
+					if (iterator_s->find(' ', 0)!=string::npos) 
 						f << "\"";
+				}
 				++iterator_s;
 				for(vector<string>::iterator its = iterator_s; its != (*itp)->value.end(); ++its)
 				{
@@ -1002,7 +1010,7 @@ void  FileDatabase :: write_file()
 						f << "\"";
 				}
 			}
-		f << endl;
+			f << endl;
 		}
 		f << endl;
 		f << "# CLASS " << (*it)->name << " attribute properties" << endl;
@@ -1242,7 +1250,8 @@ CORBA::Any*   FileDatabase :: DbPutDeviceProperty(CORBA::Any& send)
 				{
 					prop.value[j] = (*data_in)[index]; index++;
 				}
-			} else 
+			}
+			else 
 			{
 				/* it is a new property */
 				t_property* temp_property = new t_property;
@@ -1261,7 +1270,6 @@ CORBA::Any*   FileDatabase :: DbPutDeviceProperty(CORBA::Any& send)
 	
 	write_file();
 	return any_ptr;
-
 };
 
 CORBA::Any*   FileDatabase :: DbDeleteDeviceProperty(CORBA::Any& send)
@@ -1671,7 +1679,8 @@ CORBA::Any*   FileDatabase :: DbPutClassProperty(CORBA::Any& send)
 					prop.value[j] = (*data_in)[index]; index++;
 					//db_data[i] >> prop.value; 
 				}
-			} else 
+			}
+			else 
 			{
 				/* it is a new property */
 				t_property* temp_property = new t_property;
@@ -1934,15 +1943,13 @@ CORBA::Any*   FileDatabase :: DbDeleteClassAttributeProperty(CORBA::Any&)
 CORBA::Any*  FileDatabase :: DbGetDeviceList(CORBA::Any& send)
 { 
 	CORBA::Any* any_ptr = new CORBA::Any();	
-//	Tango::DevVarStringArray* data_in  = new DevVarStringArray;
 	const Tango::DevVarStringArray* data_in;
 	Tango::DevVarStringArray* data_out  = new DevVarStringArray;
 	
 	cout4 << "FILEDATABASE: entering DbGetDeviceList" << endl;
 
 	send >>= data_in;
-
-		
+	
 	if (data_in->length() == 2) 
 	{
 		if ( equalsIgnoreCase((*data_in)[0].in(), m_server.name + "/" + m_server.instance_name))
@@ -1964,6 +1971,9 @@ CORBA::Any*  FileDatabase :: DbGetDeviceList(CORBA::Any& send)
 			
 			if (i == m_server.classes.size())
 			{
+				delete any_ptr;
+				delete data_out;
+
 				TangoSys_MemStream desc;
 				desc << "File database: Can't find class " << (*data_in)[1];
 				desc << " in file " << filename << "." << ends;
@@ -1972,14 +1982,21 @@ CORBA::Any*  FileDatabase :: DbGetDeviceList(CORBA::Any& send)
 				               			(const char *)"FileDatabase::DbGetDeviceList");
 			}
 		}
+		else
+		{
+			delete any_ptr;
+			delete data_out;
+
+			TangoSys_MemStream desc;
+			desc << "File database: Can't find device server " << (*data_in)[0];
+			desc << " in file " << filename << "." << ends;
+			ApiConnExcept::throw_exception((const char *)"API_DatabaseFileError",
+				               			desc.str(),
+				               			(const char *)"FileDatabase::DbGetDeviceList");
+		}
 	}
 
-	(*any_ptr) <<= data_out;	
-	//  (*any_ptr) <<= (*data_out);
-	//CORBA::Any_var* any_var = new CORBA::Any_var;
-	//(*any_var) = any_ptr;
-	
-	//return *any_var;
+	(*any_ptr) <<= data_out;
 	return any_ptr;
 };
 
@@ -2216,7 +2233,7 @@ CORBA::Any*  FileDatabase :: DbGetProperty(CORBA::Any& send)
 	send >>= data_in;
 	
 	data_out->length(2);
-	sprintf(num_attr_str,"%lu",data_in->length()-1);
+	sprintf(num_attr_str,"%lud",data_in->length()-1);
 	(*data_out)[0] = CORBA::string_dup((*data_in)[0]);
 	(*data_out)[1] = CORBA::string_dup(zero_str);
 	
@@ -2302,5 +2319,55 @@ CORBA::Any*  FileDatabase :: DbGetAttributeAliasList(CORBA::Any&)
 };
 
 
+//-----------------------------------------------------------------------------
+//
+// method :			FileDatabase::write_event_channel_ior() -
+// 
+// description : 	Method to write the event channel ior into the file
+//
+// argument : in :	dserver : The DS process name (exec/instance)
+//					ec_ior : The event channel IOR
+//
+//-----------------------------------------------------------------------------
+
+void FileDatabase::write_event_channel_ior(string &dserver,string &ior_string)
+{
+
+//
+// Do we already have this info in file?
+//
+
+	unsigned int i;
+
+	for(i = 0; i < m_server.classes.size(); i++)
+	{
+		if ( equalsIgnoreCase (NOTIFD_CHANNEL, m_server.classes[i]->name))
+		{
+
+//
+// Yes, we have it, simply replace the old IOR by the new one (as device name!)
+//
+
+			m_server.classes[i]->devices[0]->name = ior_string;
+			break;
+		}
+	}
+			
+	if (i == m_server.classes.size())
+	{
+
+//
+// Add the pseudo notifd channel class
+//
+
+		t_device *ps_dev = new t_device;
+		ps_dev->name = ior_string;
+		t_tango_class *tg_cl = new t_tango_class;
+		tg_cl->devices.push_back(ps_dev);
+		tg_cl->name = NOTIFD_CHANNEL;
+
+		m_server.classes.push_back(tg_cl);
+	}
+}
 
 } // end of namespace Tango
