@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.26 2009/09/01 14:02:50 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.30 2010/09/09 13:44:06 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //
 // cpp 	- C++ source code file for TANGO api Connection and DeviceProxy
@@ -8,7 +8,7 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.26 2009/09/01 14:02:50 taure
 //
 // original 	- August 2002
 //
-// Copyright (C) :      2002,2003,2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2002,2003,2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -30,6 +30,21 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.26 2009/09/01 14:02:50 taure
 //
 //
 // $Log: proxy_asyn.cpp,v $
+// Revision 3.30  2010/09/09 13:44:06  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.29  2010/06/25 07:16:34  taurel
+// - Also protect the asynchronous DeviceProxy::read_attributes() methods
+// against multiple times the same attribute in att name list
+//
+// Revision 3.28  2010/04/27 07:38:03  taurel
+// - Merge with the bugfixes branch
+//
+// Revision 3.27  2009/12/18 14:51:01  taurel
+// - Safety commit before christmas holydays
+// - Many changes to make the DeviceProxy, Database and AttributeProxy
+// classes thread safe (good help from the helgrind tool from valgrind)
+//
 // Revision 3.26  2009/09/01 14:02:50  taurel
 // - Still some HP-UX code !!
 //
@@ -277,8 +292,7 @@ long Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -329,9 +343,8 @@ long Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 
 	if (faf == false)
 	{
-		id = ApiUtil::instance()->get_pasyn_table()->store_request(request,TgRequest::CMD_INOUT);
+		id = add_asyn_request(request,TgRequest::CMD_INOUT);
 		request->send_deferred();
-		pasyn_ctr++;
 	}
 	else
 	{
@@ -477,9 +490,8 @@ DeviceData Connection::command_inout_reply(long id)
 				desc << ", command " << tmp << ends;
 				CORBA::string_free(tmp);
 		
-				ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-				pasyn_ctr--;	
-				
+				remove_asyn_request(id);	
+
 				ApiCommExcept::re_throw_exception(cb_excep_mess,
 					  (const char *)"API_DeviceTimedOut",
 					  desc.str(),
@@ -512,8 +524,7 @@ DeviceData Connection::command_inout_reply(long id)
 			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 		
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;
+			remove_asyn_request(id);
 		
 			Except::re_throw_exception(ex,
 					   (const char*)"API_CommandFailed",
@@ -527,7 +538,7 @@ DeviceData Connection::command_inout_reply(long id)
 		if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
 		{
 
-			connection_state = CONNECTION_NOTOK;
+			set_connection_state(CONNECTION_NOTOK);
 		
 //
 // Re-throw nearly all CORBA system exceptions
@@ -568,8 +579,8 @@ DeviceData Connection::command_inout_reply(long id)
 //
 
 					CORBA::string_free(tmp);
-					ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-					pasyn_ctr--;
+
+					remove_asyn_request(id);
 				
 					return dd_out;
 					
@@ -582,8 +593,7 @@ DeviceData Connection::command_inout_reply(long id)
 			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 		
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;
+			remove_asyn_request(id);
 		
 			ApiCommExcept::re_throw_exception(cb_excep_mess,
 						  (const char*)"API_CommunicationFailed",
@@ -598,8 +608,7 @@ DeviceData Connection::command_inout_reply(long id)
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;
+	remove_asyn_request(id);
 				
 	return data_out;
 }
@@ -741,8 +750,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 				desc << ", command " << tmp << ends;
 				CORBA::string_free(tmp);
 				
-				ApiUtil::instance()->get_pasyn_table()->remove_request(id);			
-				pasyn_ctr--;	
+				remove_asyn_request(id);
 					
 				ApiCommExcept::re_throw_exception(cb_excep_mess,
 						  (const char *)"API_DeviceTimedOut",
@@ -775,10 +783,9 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 			
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;
+			remove_asyn_request(id);;
 			
-	               	Except::re_throw_exception(ex,
+			Except::re_throw_exception(ex,
 						   (const char*)"API_CommandFailed",
                         			   desc.str(),
 						   (const char*)"Connection::command_inout_reply()");
@@ -790,7 +797,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 		if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
 		{
 
-			connection_state = CONNECTION_NOTOK;
+			set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw all CORBA system exceptions
@@ -809,10 +816,9 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 			
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;
+			remove_asyn_request(id);
 			
-	               	ApiCommExcept::re_throw_exception(cb_excep_mess,
+			ApiCommExcept::re_throw_exception(cb_excep_mess,
 						          (const char*)"API_CommunicationFailed",
                         			          desc.str(),
 						          (const char*)"Connection::command_inout_reply()");
@@ -825,8 +831,8 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;
+	remove_asyn_request(id);
+				
 	return data_out;
 }
 
@@ -846,15 +852,13 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 
 long DeviceProxy::read_attributes_asynch(vector<string> &attr_names)
 {
-
 //
 // Reconnect to device in case it is needed
 //
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -863,7 +867,13 @@ long DeviceProxy::read_attributes_asynch(vector<string> &attr_names)
                 ApiConnExcept::re_throw_exception(e,(const char*)"API_CommandFailed",
                         desc.str(), (const char*)"DeviceProxy::read_attributes_asynch()");
 	}
-			
+
+//
+// Check that the caller did not give two times the same attribute
+//
+
+	same_att_name(attr_names,"DeviceProxy::read_attributes_asynch");
+	
 //
 // Create the request object
 //
@@ -879,8 +889,7 @@ long DeviceProxy::read_attributes_asynch(vector<string> &attr_names)
 	{
 		ClntIdent ci;
 		ApiUtil *au = ApiUtil::instance();
-		ci.cpp_clnt(au->get_client_pid());
-		
+		ci.cpp_clnt(au->get_client_pid());		
 		request = Connection::ext->device_4->_request("read_attributes_4");
 		request->add_in_arg() <<= names;
 		request->add_in_arg() <<= source;
@@ -916,9 +925,8 @@ long DeviceProxy::read_attributes_asynch(vector<string> &attr_names)
 	
 	long id = 0;
 	
-	id = ApiUtil::instance()->get_pasyn_table()->store_request(request,TgRequest::READ_ATTR);
+	id = add_asyn_request(request,TgRequest::READ_ATTR);
 	request->send_deferred();
-	pasyn_ctr++;
 	
 	return id;
 }
@@ -1001,8 +1009,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 // Remove request from request global table.
 //
 
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;	
+			remove_asyn_request(id);
 		
 			return a_ptr;
 		}
@@ -1081,8 +1088,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 // Remove request from request global table.
 //
 
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;	
+		remove_asyn_request(id);
 		
 		return dev_attr;
 	}
@@ -1163,8 +1169,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 // Remove request from request global table.
 //
 
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;	
+			remove_asyn_request(id);
 		
 			return a_ptr;
 		}
@@ -1230,8 +1235,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 // Remove request from request global table.
 //
 
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;	
+		remove_asyn_request(id);
 		
 		return dev_attr;
 	}
@@ -1346,8 +1350,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 // Remove request from request global table.
 //
 
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;	
+		remove_asyn_request(id);
 		
 		return a_ptr;
 	}
@@ -1428,9 +1431,8 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;	
-		
+	remove_asyn_request(id);	
+						
 	return dev_attr;
 }
 
@@ -1508,7 +1510,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 				break;
 			}
 		}
-		
+	
 		if (i == nb)
 		{
 			TangoSys_OMemStream desc;
@@ -1543,8 +1545,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 // Remove request from request global table.
 //
 
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;	
+		remove_asyn_request(id);
 		
 		return a_ptr;
 	}
@@ -1561,6 +1562,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 	const Tango::AttributeValueList_4 *received_4;
 	
 	CORBA::Any &dii_any = req.request->return_value();
+
 	if (version < 3)
 		dii_any >>= received;
 	else if (version == 3)
@@ -1609,12 +1611,9 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;	
-		
+	remove_asyn_request(id);
+	
 	return dev_attr;
-
-
 }
 
 //-----------------------------------------------------------------------------
@@ -1663,8 +1662,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 			}
 			desc << ends;
 			
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;	
+			remove_asyn_request(id);
 
 			if (type == SIMPLE)
 				ApiCommExcept::re_throw_exception(cb_excep_mess,
@@ -1709,8 +1707,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 		}
 		desc << ends;
 			
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;
+		remove_asyn_request(id);
 
 		if (type == SIMPLE)			
 	        	Except::re_throw_exception(ex,
@@ -1730,7 +1727,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 	if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
 	{
 
-		connection_state = CONNECTION_NOTOK;
+		set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw nearly (but not all) all CORBA system exceptions
@@ -1772,8 +1769,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 		}
 		desc << ends;
 					
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;
+		remove_asyn_request(id);
 
 		if (type == SIMPLE)
 			ApiCommExcept::re_throw_exception(cb_excep_mess,
@@ -1825,8 +1821,7 @@ long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -1887,10 +1882,9 @@ long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 //
 
 	long id = 0;		
-	id = ApiUtil::instance()->get_pasyn_table()->store_request(request,TgRequest::WRITE_ATTR);
+	id = add_asyn_request(request,TgRequest::WRITE_ATTR);
 
 	request->send_deferred();
-	pasyn_ctr++;
 		
 	return id;
 }
@@ -1916,8 +1910,7 @@ long DeviceProxy::write_attribute_asynch(DeviceAttribute &attr)
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -1976,10 +1969,9 @@ long DeviceProxy::write_attribute_asynch(DeviceAttribute &attr)
 //
 
 	long id = 0;		
-	id = ApiUtil::instance()->get_pasyn_table()->store_request(request,TgRequest::WRITE_ATTR_SINGLE);
+	id = add_asyn_request(request,TgRequest::WRITE_ATTR_SINGLE);
 
 	request->send_deferred();
-	pasyn_ctr++;
 		
 	return id;
 }
@@ -2092,8 +2084,7 @@ void DeviceProxy::write_attributes_reply(long id,long call_timeout)
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;	
+	remove_asyn_request(id);	
 }
 
 
@@ -2168,8 +2159,7 @@ void DeviceProxy::write_attributes_reply(long id)
 // Remove request from request global table.
 //
 
-	ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-	pasyn_ctr--;
+	remove_asyn_request(id);
 
 }
 
@@ -2235,8 +2225,7 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 				desc << ends;
 			}
 			
-			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-			pasyn_ctr--;	
+			remove_asyn_request(id);	
 
 			ApiCommExcept::re_throw_exception(cb_excep_mess,
 							  (const char *)"API_DeviceTimedOut",
@@ -2310,8 +2299,7 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		}
 		desc << ends;
 		
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;
+		remove_asyn_request(id);
 						
 		if (version < 3)
 		{
@@ -2352,7 +2340,7 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 	if ((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL)
 	{
 
-		connection_state = CONNECTION_NOTOK;
+		set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw nearly all CORBA system exceptions
@@ -2412,8 +2400,7 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		}
 		desc << ends;
 					
-		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
-		pasyn_ctr--;
+		remove_asyn_request(id);
 
 		ApiCommExcept::re_throw_exception(cb_excep_mess,
 						  (const char*)"API_CommunicationFailed",
@@ -2650,6 +2637,7 @@ DeviceData Connection::redo_synch_cmd(TgRequest &req)
 
 void Connection::cancel_asynch_request(long id)
 {
+	omni_mutex_lock guard(ext->asyn_mutex);
 	ApiUtil::instance()->get_pasyn_table()->mark_as_cancelled(id);
 	pasyn_ctr--;
 }
@@ -2668,6 +2656,7 @@ void Connection::cancel_asynch_request(long id)
 
 void Connection::cancel_all_polling_asynch_request()
 {
+	omni_mutex_lock guard(ext->asyn_mutex);
 	ApiUtil::instance()->get_pasyn_table()->mark_all_polling_as_cancelled();
 	pasyn_ctr = 0;
 }
