@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.18 2009/03/27 13:05:10 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.22 2010/09/12 12:18:23 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //
 // cpp 	- C++ source code file for TANGO api class and asynchronous callback
@@ -8,7 +8,7 @@ static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.18 2009/03/27 13:05:10 ta
 //
 // original 	- January 2003
 //
-// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -30,6 +30,21 @@ static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.18 2009/03/27 13:05:10 ta
 //
 //
 // $Log: proxy_asyn_cb.cpp,v $
+// Revision 3.22  2010/09/12 12:18:23  taurel
+// - Now, the test suite seems OK
+//
+// Revision 3.21  2010/09/09 13:44:06  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.20  2010/06/25 07:16:35  taurel
+// - Also protect the asynchronous DeviceProxy::read_attributes() methods
+// against multiple times the same attribute in att name list
+//
+// Revision 3.19  2009/12/18 14:51:01  taurel
+// - Safety commit before christmas holydays
+// - Many changes to make the DeviceProxy, Database and AttributeProxy
+// classes thread safe (good help from the helgrind tool from valgrind)
+//
 // Revision 3.18  2009/03/27 13:05:10  taurel
 // - Fix bug due to new Attribute format data member in AttributeValue4
 // structure
@@ -205,8 +220,7 @@ void Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -255,10 +269,9 @@ void Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 //
 
 	ApiUtil *au = ApiUtil::instance();	
-	au->get_pasyn_table()->store_request(req_seq[0],&cb,this,TgRequest::CMD_INOUT);
+	add_asyn_cb_request(req_seq[0],&cb,this,TgRequest::CMD_INOUT);
 	CORBA::ORB_ptr orb = au->get_orb();
 	orb->send_multiple_requests_deferred(req_seq);
-	pasyn_cb_ctr++;
 	if (au->get_asynch_cb_sub_model() == PUSH_CALLBACK)
 		au->get_pasyn_table()->signal();
 	
@@ -355,8 +368,7 @@ void Connection::get_asynch_replies()
 					break;
 				}
 			
-				ApiUtil::instance()->get_pasyn_table()->remove_request(this,req);
-				pasyn_cb_ctr--;
+				remove_asyn_cb_request(this,req);
 			}
 		}
 		
@@ -391,8 +403,7 @@ void Connection::get_asynch_replies()
 			break;
 		}
 		
-		ApiUtil::instance()->get_pasyn_table()->remove_request(this,tg_ptr->request);
-		pasyn_cb_ctr--;
+		remove_asyn_cb_request(this,tg_ptr->request);
 	}	
 	
 }	
@@ -530,7 +541,7 @@ void Connection::Cb_Cmd_Request(CORBA::Request_ptr req,Tango::CallBack *cb_ptr)
 			 (to_except == false))
 		{
 
-			connection_state = CONNECTION_NOTOK;
+			set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw all CORBA system exceptions
@@ -783,7 +794,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 			 (to_except == false))
 		{
 
-			connection_state = CONNECTION_NOTOK;
+			set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw all CORBA system exceptions
@@ -1026,7 +1037,7 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 			 (to_except == false))
 		{
 
-			connection_state = CONNECTION_NOTOK;
+			set_connection_state(CONNECTION_NOTOK);
 			
 //
 // Re-throw all CORBA system exceptions
@@ -1106,7 +1117,6 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 
 void Connection::get_asynch_replies(long call_timeout)
 {
-
 //
 // First check all replies already there
 //
@@ -1130,15 +1140,14 @@ void Connection::get_asynch_replies(long call_timeout)
 			break;
 		}
 		
-		ApiUtil::instance()->get_pasyn_table()->remove_request(this,tg_ptr->request);
-		pasyn_cb_ctr--;
+		remove_asyn_cb_request(this,tg_ptr->request);
 	}	
 
 //
 // If they are requests already sent but without being replied yet
 //
 
-	if (pasyn_cb_ctr != 0)
+	if (get_pasyn_cb_ctr() != 0)
 	{	
 		CORBA::ORB_ptr orb = ApiUtil::instance()->get_orb();
 		CORBA::Request_ptr req;
@@ -1158,7 +1167,7 @@ void Connection::get_asynch_replies(long call_timeout)
 			to_wait.tv_nsec = 20000000;
 #endif				
 				
-			while ((nb > 0) && (pasyn_cb_ctr != 0))
+			while ((nb > 0) && (get_pasyn_cb_ctr() != 0))
 			{
 #ifdef _TG_WINDOWS_
 				Sleep(20);
@@ -1202,8 +1211,7 @@ void Connection::get_asynch_replies(long call_timeout)
 							break;
 						}
 			
-						ApiUtil::instance()->get_pasyn_table()->remove_request(this,req);
-						pasyn_cb_ctr--;
+						remove_asyn_cb_request(this,req);
 					}
 				}
 			}
@@ -1213,7 +1221,7 @@ void Connection::get_asynch_replies(long call_timeout)
 // without replies
 //
 			
-			if ((nb == 0) && (pasyn_cb_ctr != 0))
+			if ((nb == 0) && (get_pasyn_cb_ctr() != 0))
 			{	
 				TangoSys_OMemStream desc;
 				desc << "Still some reply(ies) for asynchronous callback call(s) to be received" << ends;
@@ -1230,7 +1238,7 @@ void Connection::get_asynch_replies(long call_timeout)
 // device has sent their replies
 //
 
-			while (pasyn_cb_ctr != 0)
+			while (get_pasyn_cb_ctr() != 0)
 			{					
 				orb->get_next_response(req);
 								
@@ -1265,8 +1273,7 @@ void Connection::get_asynch_replies(long call_timeout)
 						break;
 					}
 			
-					ApiUtil::instance()->get_pasyn_table()->remove_request(this,req);
-					pasyn_cb_ctr--;
+					remove_asyn_cb_request(this,req);
 				}
 			}
 		}
@@ -1296,8 +1303,7 @@ void DeviceProxy::read_attributes_asynch(vector<string> &attr_names,CallBack &cb
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -1306,6 +1312,12 @@ void DeviceProxy::read_attributes_asynch(vector<string> &attr_names,CallBack &cb
                 ApiConnExcept::re_throw_exception(e,(const char*)"API_CommandFailed",
                         desc.str(), (const char*)"DeviceProxy::read_attributes_asynch()");
 	}
+
+//
+// Check that the caller did not give two times the same attribute
+//
+
+	same_att_name(attr_names,"DeviceProxy::read_attributes_asynch");
 	
 //
 // Create the request object
@@ -1360,10 +1372,9 @@ void DeviceProxy::read_attributes_asynch(vector<string> &attr_names,CallBack &cb
 //
 
 	ApiUtil *au = ApiUtil::instance();	
-	au->get_pasyn_table()->store_request(req_seq[0],&cb,this,TgRequest::READ_ATTR);
+	add_asyn_cb_request(req_seq[0],&cb,this,TgRequest::READ_ATTR);
 	CORBA::ORB_ptr orb = au->get_orb();
 	orb->send_multiple_requests_deferred(req_seq);
-	pasyn_cb_ctr++;
 	if (au->get_asynch_cb_sub_model() == PUSH_CALLBACK)
 		au->get_pasyn_table()->signal();
 }
@@ -1397,8 +1408,7 @@ void DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list,
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -1463,10 +1473,9 @@ void DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list,
 //
 
 	ApiUtil *au = ApiUtil::instance();	
-	au->get_pasyn_table()->store_request(req_seq[0],&cb,this,TgRequest::WRITE_ATTR);
+	add_asyn_cb_request(req_seq[0],&cb,this,TgRequest::WRITE_ATTR);
 	CORBA::ORB_ptr orb = au->get_orb();
 	orb->send_multiple_requests_deferred(req_seq);
-	pasyn_cb_ctr++;
 	if (au->get_asynch_cb_sub_model() == PUSH_CALLBACK)
 		au->get_pasyn_table()->signal();
 }
@@ -1501,8 +1510,7 @@ void DeviceProxy::write_attribute_asynch(DeviceAttribute &attr,CallBack &cb)
 
 	try
 	{
-		if (connection_state != CONNECTION_OK)
-			reconnect(dbase_used);
+		check_and_reconnect();
 	}
 	catch (Tango::ConnectionFailed &e)
 	{
@@ -1564,10 +1572,9 @@ void DeviceProxy::write_attribute_asynch(DeviceAttribute &attr,CallBack &cb)
 //
 
 	ApiUtil *au = ApiUtil::instance();	
-	au->get_pasyn_table()->store_request(req_seq[0],&cb,this,TgRequest::WRITE_ATTR_SINGLE);
+	add_asyn_cb_request(req_seq[0],&cb,this,TgRequest::WRITE_ATTR_SINGLE);
 	CORBA::ORB_ptr orb = au->get_orb();
 	orb->send_multiple_requests_deferred(req_seq);
-	pasyn_cb_ctr++;
 	if (au->get_asynch_cb_sub_model() == PUSH_CALLBACK)
 		au->get_pasyn_table()->signal();
 }
