@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attrdesc.cpp,v 3.15 2009/09/16 12:16:19 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: attrdesc.cpp,v 3.17 2010/09/09 13:44:46 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+============================================================================
 //
@@ -16,7 +16,7 @@ static const char *RcsId = "$Id: attrdesc.cpp,v 3.15 2009/09/16 12:16:19 taurel
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -36,9 +36,16 @@ static const char *RcsId = "$Id: attrdesc.cpp,v 3.15 2009/09/16 12:16:19 taurel
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.15 $
+// $Revision: 3.17 $
 //
 // $Log: attrdesc.cpp,v $
+// Revision 3.17  2010/09/09 13:44:46  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.16  2009/12/09 15:48:56  taurel
+// - Add Attr and Attribute methods set_data_ready_event() and is_data_ready_event().
+// - Admin device command EventSubcriptionChange fails is one of these methods has not been called
+//
 // Revision 3.15  2009/09/16 12:16:19  taurel
 // - Better management of the RDS alarm properties. They were not taken into
 // account by the set_default_properties() method
@@ -245,6 +252,7 @@ Attr::Attr(const char *att_name,long att_type,AttrWriteType att_writable,
 	ext->check_change_event = true;
 	ext->fire_archive_event = false;
 	ext->check_archive_event = true;
+	ext->fire_dr_event = false;
 	
 	if (name != "State")
 		check_type();
