@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_3.cpp,v 3.50 2009/11/09 12:04:31 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: device_3.cpp,v 3.55 2010/09/30 14:16:52 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+============================================================================
 //
@@ -15,7 +15,7 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.50 2009/11/09 12:04:31 taurel
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -35,9 +35,31 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.50 2009/11/09 12:04:31 taurel
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.50 $
+// $Revision: 3.55 $
 //
 // $Log: device_3.cpp,v $
+// Revision 3.55  2010/09/30 14:16:52  taurel
+// - Do not overwrite WAttribute written value if the user set it in its
+// write_xxx method
+//
+// Revision 3.54  2010/09/09 13:45:22  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.53  2010/09/09 13:29:09  taurel
+// - Commit after the last merge with the bugfixes branch
+// - Fix some warning when compiled -W -Wall
+//
+// Revision 3.52  2010/06/21 14:01:15  taurel
+// - Yet another merge with the Release_7_1_1-bugfixes branch
+//
+// Revision 3.51  2010/02/22 13:00:33  taurel
+// - Add a better exception message in case of major error in the
+// attribute set_value() method (for instance in case the passed pointer is
+// NULL)
+// Revision 3.50.2.1  2010/06/21 13:26:27  taurel
+// - Fix possible deadlock due to attribute mutex management.
+// This is a SourceForge bug
+//
 // Revision 3.50  2009/11/09 12:04:31  taurel
 // - The attribute mutex management is in the AttributeValue_4 struct
 //
@@ -777,7 +799,11 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 					{
 						cout4 << "Locking attribute mutex for attribute " << att.get_name() << endl;
 						omni_mutex *attr_mut = att.get_attr_mutex();
-						attr_mut->lock();
+						if (attr_mut->trylock() == 0)
+						{
+							cout4 << "Mutex for attribute " << att.get_name() << " is already taken.........." << endl;
+							attr_mut->lock();
+						}
 					}
 					
 //
@@ -818,6 +844,45 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 						clear_att_dim((*back4)[index]);					
 					}
 				}
+				catch (...)
+				{
+					long index;
+					if (second_try == false)
+						index = wanted_attr[i].idx_in_names;
+					else
+						index = idx[wanted_attr[i].idx_in_names];
+
+					wanted_attr[i].failed = true;
+					Tango::DevErrorList del;
+					del.length(1);
+
+					del[0].severity = Tango::ERR;
+					del[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_no_except");
+					del[0].reason = CORBA::string_dup("API_CorbaSysException ");
+					del[0].desc = CORBA::string_dup("Unforseen exception when trying to read attribute. It was even not a Tango DevFailed exception");
+
+					if (back != NULL)
+					{
+						(*back)[index].err_list = del;
+						(*back)[index].quality = Tango::ATTR_INVALID;
+						(*back)[index].name = CORBA::string_dup(names[wanted_attr[i].idx_in_names]);
+						clear_att_dim((*back)[index]);
+					}
+					else
+					{							
+						if ((att.get_attr_serial_model() == ATTR_BY_KERNEL) && (is_allowed_failed == false))
+						{
+							cout4 << "Releasing attribute mutex for attribute " << att.get_name() << " due to a severe error which is not a DevFailed" << endl;
+							omni_mutex *attr_mut = att.get_attr_mutex();
+							attr_mut->unlock();
+						}
+						
+						(*back4)[index].err_list = del;
+						(*back4)[index].quality = Tango::ATTR_INVALID;
+						(*back4)[index].name = CORBA::string_dup(names[wanted_attr[i].idx_in_names]);
+						clear_att_dim((*back4)[index]);	
+					}
+				}
 			}
 		}
 
@@ -1203,7 +1268,9 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 							{
 								cout4 << "Asking CORBA structure to release attribute mutex for attribute " << att.get_name() << endl;
 								if (att.get_writable() != Tango::WRITE)
+								{
 									REL_ATT_MUTEX(back4,index,att);
+								}
 								
 								(*back4)[index].err_list = e.errors;
 								(*back4)[index].quality = Tango::ATTR_INVALID;
@@ -1908,12 +1975,14 @@ void Device_3Impl::write_attributes_34(const Tango::AttributeValueList *values_3
 //  During device access inside the same server,
 //  the thread stays the same!
 //
+	
 	SubDevDiag &sub = (Tango::Util::instance())->get_sub_dev_diag();
 	string last_associated_device = sub.get_associated_device();
 	sub.set_associated_device(get_name());
 
 // Catch all execeptions to set back the associated device after
 // execution
+	
 	try
 	{
 
@@ -2096,6 +2165,7 @@ void Device_3Impl::write_attributes_34(const Tango::AttributeValueList *values_3
 				{
 					WAttribute &att = dev_attr->get_w_attr_by_ind((*ite).idx_in_multi_attr);
 					att.set_value_flag(false);
+					att.set_user_set_write_value(false);
 					vector<Tango::Attr *> &attr_vect = device_class->get_class_attr()->get_attr_list();
 					if (attr_vect[att.get_attr_idx()]->is_allowed(this,Tango::WRITE_REQ) == false)
 					{
@@ -2140,6 +2210,9 @@ void Device_3Impl::write_attributes_34(const Tango::AttributeValueList *values_3
 // Copy data into Attribute object, store the memorized one in db
 // and if the attribute has a RDS alarm, set the write date
 //
+// Warning: Do not copy caller value if the user has manually set the
+// attribute written value in its write method
+//
 // WARNING: --> The DevEncoded data type is suported only as SCALAR and is not
 // memorizable. Therefore, no need to call copy_data
 //
@@ -2151,15 +2224,20 @@ void Device_3Impl::write_attributes_34(const Tango::AttributeValueList *values_3
 			WAttribute &att = dev_attr->get_w_attr_by_ind(updated_attr[i].idx_in_multi_attr);
 
 			if (values_3 != NULL)
-				att.copy_data((*values_3)[updated_attr[i].idx_in_names].value);
+			{
+				if (att.get_user_set_write_value() == false)
+					att.copy_data((*values_3)[updated_attr[i].idx_in_names].value);
+			}
 			else
-				att.copy_data((*values_4)[updated_attr[i].idx_in_names].value);
+			{
+				if (att.get_user_set_write_value() == false)
+					att.copy_data((*values_4)[updated_attr[i].idx_in_names].value);
+			}
 
 			if (att.is_memorized() == true)
 				att_in_db.push_back(i);
 			if (att.is_alarmed().test(Attribute::rds) == true)
-				att.set_written_date();			
-
+				att.set_written_date();
 		}
 
 		if ((Tango::Util::_UseDb == true) && (att_in_db.size() != 0))
