@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp,v 1.33 2009/09/18 09:18:06 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: deviceclass.cpp,v 1.37 2010/09/24 14:06:15 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+============================================================================
 //
@@ -13,7 +13,7 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.33 2009/09/18 09:18:06 taur
 //
 // author(s) :		E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -33,9 +33,22 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.33 2009/09/18 09:18:06 taur
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 1.33 $
+// $Revision: 1.37 $
 //
 // $Log: deviceclass.cpp,v $
+// Revision 1.37  2010/09/24 14:06:15  taurel
+// - For Python DS, do not give full device ownership to the POA.
+// Otherwise, a python DS crashes at exit.
+//
+// Revision 1.36  2010/09/09 13:45:22  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 1.35  2010/06/23 09:13:28  taurel
+// - Change some comments
+//
+// Revision 1.34  2010/06/21 12:38:23  taurel
+// - Implement a much faster server shutdown sequence
+//
 // Revision 1.33  2009/09/18 09:18:06  taurel
 // - End of attribute serialization implementation?
 //
@@ -595,10 +608,12 @@ void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 							att_val[nb_wr - 1].value <<= uch_seq;
 							break;
 						}
-						
+
+//						
 // Check the initialisation flag for memorized attributes.
 // The the flag is false, do not add the element to the att_val
 // vector. This avoids a call to write the memorozied value to the attribute.
+//
 					
 					if ( att.is_memorized_init() == false )
 						{
@@ -841,71 +856,23 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 // Deactivate the CORBA object
 //
 
-	if (device_list[idx]->get_exported_flag() == true)
+	bool py_dev = device_list[idx]->is_py_device();
+	bool exported_device = device_list[idx]->get_exported_flag();
+
+	if (exported_device == true)
 		r_poa->deactivate_object(device_list[idx]->get_obj_id().in());
 			
 //
-// Unfortunately, the deactivate_object on the POA immediately returns even if there
-// is a running call (see Henning/Vinoski page 500). Therefore, try to get the
-// object monitor which will be free only when the last running call will finish
-// Two cases have to be taken into account :
-// 1 - The running commands : The get/rel on the monitor will make the thread
-//     waiting for them
-// 2- BUT, there are also (for very demanding clients) cases where commands are
-//    registered by the ORB but the monitor owner. For this case, sleep a while
-//    to give them time to execute. But sleep how long ?
-//    By default, I choose 200 mS
-// May be there is a more CORBA way to do this.
-// Need to investigate
-//
-	omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
-	PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;		
-
-#ifdef _TG_WINDOWS_
-	try
-	{
-		lock_ptr->Release();
-
-		device_list[idx]->get_dev_monitor().get_monitor();	
-		device_list[idx]->get_dev_monitor().rel_monitor();
-		
-		Sleep(200);
-
-		lock_ptr->Get();
-	}
-#else
-	try
-	{
-		lock_ptr->Release();
-
-		device_list[idx]->get_dev_monitor().get_monitor();
-		device_list[idx]->get_dev_monitor().rel_monitor();
-
-		struct timespec wait_to_sleep,still_wait;
-		wait_to_sleep.tv_sec = 0;
-		wait_to_sleep.tv_nsec = 200000000;
-
-		nanosleep(&wait_to_sleep,&still_wait);
-
-		lock_ptr->Get();
-	}
-#endif /* _TG_WINDOWS_ */
-	catch (Tango::DevFailed &)
-	{
-	}
-		
-//
-// Remove the C++ servant
+// Remove the servant.
+// For C++ device, this will be automatically done by the POA when the last executing call
+// will be over.
 //
 
-	if (device_list[idx]->is_py_device() == false)
+	if (py_dev == false)
 	{
-		if (device_list[idx]->get_exported_flag() == true)
-			device_list[idx]->_remove_ref();
-		else
+		if (exported_device == false)
 		{
-			Device_3Impl *dev_3 = static_cast<Device_3Impl *>(device_list[idx]);
-			dev_3->delete_dev();
+			delete device_list[idx];
 		}
 	}		
 	else
@@ -1020,10 +987,13 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 			
 // 
 // Activate the CORBA object incarnated by the dev C++ object
+// Also call _remove_ref to give POA the full ownership of servant
 //
 
 		d = dev->_this();
 		dev->set_d_var(Tango::Device::_duplicate(d));
+		if (is_py_class() == false)
+			dev->_remove_ref();
 
 //
 // Store the ObjectId (The ObjectId_var type is a typedef of a string_var
@@ -1082,7 +1052,8 @@ void DeviceClass::export_device(DeviceImpl *dev,const char *corba_obj_name)
 		
 		d = dev->_this();
 		dev->set_obj_id(id);		
-		dev->set_d_var(Tango::Device::_duplicate(d));		
+		dev->set_d_var(Tango::Device::_duplicate(d));
+		dev->_remove_ref();	
 	}
 			
 //
