@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverpoll.cpp,v 3.31 2009/10/23 14:36:27 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: dserverpoll.cpp,v 3.36 2010/09/09 13:45:22 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+=============================================================================
 //
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.31 2009/10/23 14:36:27 taur
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -34,9 +34,29 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.31 2009/10/23 14:36:27 taur
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.31 $
+// $Revision: 3.36 $
 //
 // $Log: dserverpoll.cpp,v $
+// Revision 3.36  2010/09/09 13:45:22  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.35  2010/07/16 10:53:13  taurel
+// - Even with a minimun polling period defined, i is possible to ask for a
+// polling period of 0 (externally trigged or polling buffer externally filled in)
+//
+// Revision 3.34  2010/06/18 13:57:09  taurel
+// - Add a way (using properties) to define a minimum polling period
+//
+// Revision 3.33  2010/06/18 07:45:47  taurel
+// - In case of locked device, polling and logging related commands are
+// allowed only for the locker process
+//
+// Revision 3.32  2010/04/27 07:36:42  taurel
+// - Merge with the bugfixes branch
+//
+// Revision 3.31.2.1  2010/03/31 06:42:43  taurel
+// - Fix a case dependency bug when updating polling related property.
+//
 // Revision 3.31  2009/10/23 14:36:27  taurel
 // - Tango 7.1.1
 // - Fix bugs 2880372 and 2881841
@@ -100,221 +120,6 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.31 2009/10/23 14:36:27 taur
 // - The polling thread is not configured by a separate thread any more.
 // The Add_obj_polling command now support a delta_t to start the first polling
 //
-// Revision 3.14  2007/05/04 13:02:18  jensmeyer
-// Added the removal of the attribute polling and event configuration when
-// executing the remove_attribute() method.
-//
-// Revision 3.13  2007/04/20 14:40:33  taurel
-// - Ported to Windows 64 bits x64 architecture
-//
-// Revision 3.12  2007/04/16 14:56:36  taurel
-// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
-// - Ported to omniORB4.1
-// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
-// - Added a new filterable field in the archive event
-//
-// Revision 3.11  2007/03/29 07:00:28  taurel
-// - Fix some data type for 64 bits compatibility
-//
-// Revision 3.10  2006/04/13 13:31:44  taurel
-// Several changes:
-// - A new DeviceImpl::push_event() method to push USER event with State as data
-// - Fix bug in polling thread POLL_IUPD_PERIOD. The new polling period was applied only after the following poll still done with the old polling period
-// - It is now possible to update polling period of polled objects from a polled attribute or command
-//
-// Revision 3.9  2006/01/20 08:12:17  taurel
-// - No real change in this file. Simply changes in indentation
-//
-// Revision 3.8  2005/06/29 08:31:17  taurel
-// - Last commit before release 5.2 ?
-//
-// Revision 3.7  2005/01/13 08:29:05  taurel
-// - Merge trunk with Release_5_0 from brach Release_5_branch
-//
-// Revision 3.6.2.8  2004/12/06 14:39:29  taurel
-// - Polling starts in a separate thread
-// - For Windows : Polling thread cmd/attr measurement used PerformanceCounter
-// - Fix bug in pollext.h in methods to externally fill polling buffer
-//
-// Revision 3.6.2.7  2004/11/04 09:46:38  taurel
-// - Add a tuning method in the polling thread
-// - Some minor fixes to pass test suite
-//
-// Revision 3.6.2.6  2004/10/22 11:25:50  taurel
-// Added warning alarm
-// Change attribute config. It now includes alarm and event parameters
-// Array attribute property now supported
-// subscribe_event throws exception for change event if they are not correctly configured
-// Change in the polling thread: The event heartbeat has its own work in the work list
-// Also add some event_unregister
-// Fix order in which classes are destructed
-// Fix bug in asynchronous mode (PUSH_CALLBACK). The callback thread ate all the CPU
-// Change in the CORBA info call for the device type
-//
-// Revision 3.6.2.5  2004/09/15 06:46:28  taurel
-// - Added four new types for attributes (boolean, float, unsigned short and unsigned char)
-// - It is also possible to read state and status as attributes
-// - Fix bug in Database::get_class_property() method (missing ends insertion)
-// - Fix bug in admin device DevRestart command (device name case problem)
-//
-// Revision 3.6.2.4  2004/08/26 07:34:45  taurel
-// - Implement a way to directly fills command or attribute polling buffer
-//
-// Revision 3.6.2.3  2004/08/19 07:44:59  taurel
-// - Replace server low level database access call by Database class method call
-// - Split device monitor in 3 : 1 to protect harware access, 1 to protect cache access and one mutex for device black box
-//
-// Revision 3.6.2.2  2004/07/15 15:04:06  taurel
-// - Added the way to externally filled the polling buffer for attribute
-//   (Command will come soon)
-//
-// Revision 3.6.2.1  2004/07/08 11:02:13  taurel
-// - Add possibility to have command/attribute polling buffer depth specified
-//   individually
-//
-// Revision 3.6  2004/07/07 08:40:11  taurel
-//
-// - Fisrt commit after merge between Trunk and release 4 branch
-// - Add EventData copy ctor, asiignement operator and dtor
-// - Add Database and DeviceProxy::get_alias() method
-// - Add AttributeProxy ctor from "device_alias/attribute_name"
-// - Exception thrown when subscribing two times for exactly yhe same event
-//
-// Revision 3.5  2003/08/22 12:52:54  taurel
-// - For device implementing release 3 of IDL (derivating from device_3impl), change
-//   the way how attributes are read or written
-// - Fix small bug in vector iterator usage in case of "erase()" method called in
-//   a for loop
-//
-// Revision 3.4  2003/08/21 07:23:46  taurel
-// - End of the implementation of the new way to transfer data for read and
-//   write attributes (better use of exception)
-// - Added Attribute::set_date() and Attribute::set_value_date_quality() methods
-// - Added DeviceAttribute ctors from "const char *"
-// - Enable writing of spectrum and image attributes
-// - Many new DeviceAttribute ctors/inserters to enable easy image and spectrums
-//   attribute writing
-// - Attribute date automatically set in case of attribute quality factor set to INVALID
-// - Change in the polling thread discarding element algo. to support case of polling
-//   several cmd/atts at the same polling period with cmd/attr having a long response time
-// - Take cmd/attr execution time into account in the "Data not updated since" polling
-//   status string
-// - Split "str().c_str()" code in two lines of code. It was the reason of some problem
-//   on Windows device server
-// - Add the possibility to set a cmd/attr polling as "externally triggered". Add method
-//   to send trigger to the polling thread
-//
-// Revision 3.3.2.3  2004/03/09 16:36:36  taurel
-// - Added HP aCC port (thanks to Claudio from Elettra)
-// - Some last small bugs fixes
-//
-// Revision 3.3.2.2  2003/10/03 13:34:26  taurel
-// - Fix bug for device server started without database, with device name specified on command line using mix of upper and lower cases
-// - It's now possible to send a command to the polling thread from itself
-// - Add a release lock if the insert into the polling buffer failed
-//
-// Revision 3.3.2.1  2003/09/30 11:50:43  taurel
-// Add some changes foreseen for release 4.1 and already implemented on
-// the trunck into this release 4.0 branch
-//
-// Revision 3.3  2003/05/28 14:55:09  taurel
-// Add the include (conditionally) of the include files generated by autoconf
-//
-// Revision 3.2  2003/05/16 08:46:16  taurel
-// Many changes for release 3.0.1. The most important ones are :
-// - Timeout are backs
-// - Multiple db servers (change in TANGO_HOST syntax)
-// - Added methods to print DeviceData, DeviceDataHistory, DeviceAttribute and DeviceAttributeHistory instances
-// - Attributes name stored in blackbox
-// - Remove check if a class is created without any device
-// - It's now possible to create a DeviceProxy from its alias name
-// - Command, attribute names are case insensitive
-// - Change parameters of some DeviceProxy logging methods
-// - Change parameters of DeviceProxy asynchronous replies calls
-// - New serialization model in device server (no serialization model)
-// - Win32 (2000) device server service does not exit at loggoff anymore
-// - Miscellaneous bug fixes
-//
-// Revision 3.1  2003/04/03 15:24:09  taurel
-// Added methods to print DeviceData, DeviceAttribute, DeviceDataHistory
-// and DeviceAttributeHistory instance
-//
-// Revision 3.0  2003/03/25 16:43:02  taurel
-// Many changes for Tango release 3.0 including
-// - Added full logging features
-// - Added asynchronous calls
-// - Host name of clients now stored in black-box
-// - Three serialization model in DS
-// - Fix miscellaneous bugs
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and destructor methods
-// - Some internal cleanups
-// - Change the way how TangoMonitor class is implemented. It's a recursive
-//   mutex
-//
-// Revision 2.9  2003/01/09 12:03:16  taurel
-// - Ported to gcc 3.2
-// - Added ApiUtil::cleanup() and ApiUtil::~ApiUtil() methods
-// - Replace some ORB * by ORB_ptr
-// - Use CORBA::ORB::is_nil() instead of comparing to NULL
-//
-// Revision 2.8  2002/12/16 12:07:19  taurel
-// No change in code at all but only forgot th emost important line in
-// list of updates in the previous release :
-// - Change underlying ORB from ORBacus to omniORB
-//
-// Revision 2.7  2002/12/16 10:16:21  taurel
-// - New method get_device_list() in Util class
-// - Util::get_class_list takes DServer device into account
-// - Util::get_device_by_name() takes DServer device into account
-// - Util::get_device_list_by_class() takes DServer device into account
-// - New parameter to the attribute::set_value() method to enable CORBA to free
-// memory allocated for the attribute
-//
-// Revision 2.6  2002/10/17 07:43:06  taurel
-// Fix bug in history stored by the polling thread :
-// - We need one copy of the attribute data to build an history!!! It is true
-// also for command which return data created by the DeviceImpl::create_xxx
-// methods. Chnage in pollring.cpp/pollring.h/dserverpoll.cpp/pollobj.cpp
-// and pollobj.h
-//
-// Revision 2.5  2002/10/15 11:27:19  taurel
-// Fix bugs in device.cpp file :
-// - Protect the state and status CORBA attribute with the device monitor
-// Add the "TgLibVers" string as a #define in tango_config.h
-//
-// Revision 2.4  2002/08/12 15:06:54  taurel
-// Several big fixes and changes
-//   - Remove HP-UX specific code
-//   - Fix bug in polling alogorithm which cause the thread to enter an infinite
-//     loop (pollthread.cpp)
-//   - For bug for Win32 device when trying to set attribute config
-//     (attribute.cpp)
-//
-// Revision 2.3  2002/07/02 15:22:24  taurel
-// Miscellaneous small changes/bug fixes for Tango CPP release 2.1.0
-//     - classes reference documentation now generated using doxygen instead of doc++
-//     - A little file added to the library which summarizes version number.
-//       The RCS/CVS "ident" command will now tells you that release library x.y.z is composed
-//       by C++ client classes set release a.b and C++ server classes set release c.d
-//     - Fix incorrect field setting for DevFailed exception re-thrown from a CORBA exception
-//     - It's now not possible to poll the Init command
-//     - It's now possible to define a default class doc. per control system
-//       instance (using property)
-//     - The test done to check if attribute value has been set before it is
-//       returned to caller is done only if the attribute quality is set to VALID
-//     - The JTCInitialize object is now stored in the Util
-//     - Windows specific : The tango.h file now also include winsock.h
-//
-// Revision 2.2  2002/04/30 10:50:41  taurel
-// Don't check alarm on attribute if attribute quality factor is INVALID
-//
-// Revision 2.1  2002/04/29 12:24:03  taurel
-// Fix bug in attribute::set_value method and on the check against min and max value when writing attributes
-//
-// Revision 2.0  2002/04/09 14:45:10  taurel
-// See Tango WEB pages for list of changes
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
@@ -839,6 +644,13 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 				   	   (const char *)"DServer::add_obj_polling");
 	}
 
+//
+// If the device is locked and if the client is not the lock owner,
+// refuse to do the job
+//
+
+	check_lock_owner(dev,"add_obj_polling",(argin->svalue)[0]);
+
 //
 // Check that the command (or the attribute) exists. For command, also checks
 // that it does not need input value.
@@ -930,6 +742,15 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 		Except::throw_exception((const char *)"API_NotSupported",o.str(),
 					(const char *)"DServer::add_obj_polling");
 	}
+
+//
+// Check that the requested polling period is not below the one authorized
+// (if defined)
+// 0 as polling period is always authorized for polling buffer externally filled
+//
+
+	if (upd != 0)
+		check_upd_authorized(dev,upd,type,obj_name);
 	
 //
 // Round the update period (which is in mS) to the next tenth of a second
@@ -1065,7 +886,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 			vector<string>::iterator ite;
 			for (ite = non_auto_list.begin();ite < non_auto_list.end();++ite)
 			{
-				if (*ite == obj_name)
+				if (TG_strcasecmp((*ite).c_str(),obj_name.c_str()) == 0)
 				{
 					non_auto_list.erase(ite);
 					db_info.name = "non_auto_polled_cmd";
@@ -1099,7 +920,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 			vector<string>::iterator ite;
 			for (ite = non_auto_list.begin();ite < non_auto_list.end();++ite)
 			{
-				if (*ite == obj_name)
+				if (TG_strcasecmp((*ite).c_str(),obj_name.c_str()) == 0)
 				{
 					non_auto_list.erase(ite);
 					db_info.name = "non_auto_polled_attr";
@@ -1250,6 +1071,13 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		Except::throw_exception((const char *)"API_DeviceNotPolled",o.str(),
 				   	(const char *)"DServer::upd_obj_polling_period");
 	}
+
+//
+// If the device is locked and if the client is not the lock owner,
+// refuse to do the job
+//
+
+	check_lock_owner(dev,"upd_obj_polling_period",(argin->svalue)[0]);
 	
 //
 // Find the wanted object in the list of device polled object
@@ -1287,6 +1115,14 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		
 	vector<PollObj *>::iterator ite = dev->get_polled_obj_by_type_name(type,obj_name);
 
+//
+// Check that the requested polling period is not below the one authorized
+// (if defined)
+//
+
+	int upd = (argin->lvalue)[0];
+	check_upd_authorized(dev,upd,type,obj_name);
+
 //
 // Check that it is not an externally triggered polling object. In this
 // case, throw exception
@@ -1314,7 +1150,6 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 // Check that the update period is not to small
 //
 
-	int upd = (argin->lvalue)[0];
 	if ((upd != 0) && (upd < MIN_POLL_PERIOD))
 	{
 		TangoSys_OMemStream o;
@@ -1415,7 +1250,7 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 			vector<string> &cmd_list = dev->get_polled_cmd();
 			for (i = 0;i < cmd_list.size();i = i+2)
 			{
-				if (cmd_list[i] == obj_name)
+				if (TG_strcasecmp(cmd_list[i].c_str(),obj_name.c_str()) == 0)
 				{
 					cmd_list[i + 1] = upd_str;
 					break;
@@ -1433,7 +1268,7 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 			vector<string> &attr_list = dev->get_polled_attr();
 			for (i = 0;i < attr_list.size();i = i+2)
 			{
-				if (attr_list[i] == obj_name)
+				if (TG_strcasecmp(attr_list[i].c_str(),obj_name.c_str()) == 0)
 				{
 					attr_list[i + 1] = upd_str;
 					break;
@@ -1520,6 +1355,13 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 		Except::throw_exception((const char *)"API_DeviceNotPolled",o.str(),
 				   	(const char *)"DServer::rem_obj_polling");
 	}
+
+//
+// If the device is locked and if the client is not the lock owner,
+// refuse to do the job
+//
+
+	check_lock_owner(dev,"rem_obj_polling",(*argin)[0]);
 	
 //
 // Find the wanted object in the list of device polled object
@@ -1669,7 +1511,7 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 			vector<string>::iterator s_ite;	
 			for (s_ite = cmd_list.begin();s_ite < cmd_list.end();++s_ite)
 			{
-				if (*s_ite == obj_name)
+				if (TG_strcasecmp((*s_ite).c_str(),obj_name.c_str()) == 0)
 				{
 					s_ite = cmd_list.erase(s_ite);
 					cmd_list.erase(s_ite);
@@ -1684,7 +1526,7 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 				vector<string> &non_auto_cmd = dev->get_non_auto_polled_cmd();
 				for (s_ite = non_auto_cmd.begin();s_ite < non_auto_cmd.end();s_ite++)
 				{
-					if (*s_ite == obj_name)
+					if (TG_strcasecmp((*s_ite).c_str(),obj_name.c_str()) == 0)
 						break;
 				}
 				if (s_ite == non_auto_cmd.end())
@@ -1702,12 +1544,12 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 			vector<string>::iterator s_ite;	
 			for (s_ite = attr_list.begin();s_ite < attr_list.end();++s_ite)
 			{
-				if (*s_ite == obj_name)
+				if (TG_strcasecmp((*s_ite).c_str(),obj_name.c_str()) == 0)
 				{
 					s_ite = attr_list.erase(s_ite);
 					attr_list.erase(s_ite);
 					db_info << attr_list;
-					update_needed = true;			
+					update_needed = true;
 					break;
 				}
 				++s_ite;
@@ -1717,7 +1559,7 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 				vector<string> &non_auto_attr = dev->get_non_auto_polled_attr();
 				for (s_ite = non_auto_attr.begin();s_ite < non_auto_attr.end();s_ite++)
 				{
-					if (*s_ite == obj_name)
+					if (TG_strcasecmp((*s_ite).c_str(),obj_name.c_str()) == 0)
 						break;
 				}
 				if (s_ite == non_auto_attr.end())
@@ -2114,5 +1956,86 @@ void DServer::rem_event_heartbeat()
 	cout4 << "Thread cmd normally executed" << endl;
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DServer::check_upd_authorized()
+// 
+// description : 	In case a minimun update polling period is defined (via the
+//					min_poll_period, cmd_min_poll_period,attr_min_poll_period)
+//					check if the requested period is not smaller
+//
+// argin: dev : The device
+//		  upd : The requested update period
+//		  type : The polled object type (cmd / attr)
+//		  obj_name : The polled object name
+//
+//-----------------------------------------------------------------------------
+
+void DServer::check_upd_authorized(DeviceImpl *dev,int upd,PollObjType obj_type,string &obj_name)
+{
+	int min_upd = 0;
+
+//
+// Get first the xxx_min_poll_period then if not defined the min_poll_period
+//
+
+	vector<string> *v_ptr;
+	if (obj_type == Tango::POLL_CMD)
+		v_ptr = &(dev->get_cmd_min_poll_period());
+	else
+		v_ptr = &(dev->get_attr_min_poll_period());
+
+	vector<string>::iterator ite = find(v_ptr->begin(),v_ptr->end(),obj_name);
+	if (ite != v_ptr->end())
+	{
+cout << obj_name << " found in vector" << endl;
+		ite++;
+		TangoSys_MemStream s;
+		s << *ite;
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
+		if ((s >> min_upd) == false)
+#else
+		s >> min_upd;
+		if (!s)
+#endif
+		{
+			TangoSys_OMemStream o;
+			o << "System property ";
+			if (obj_type == Tango::POLL_CMD)
+				o << "cmd_min_poll_period";
+			else
+				o << "attr_min_poll_period";
+			o << " for device " << dev->get_name() << " has wrong syntax" << ends;
+			Except::throw_exception((const char *)"API_BadConfigurationProperty",
+				        			o.str(),
+				        			(const char *)"DServer::check_upd_uthorized()");
+		}
+	}
+	else
+	{
+cout << "min_upd from min_poll_period" << endl;
+		min_upd = dev->get_min_poll_period();
+	}
+
+//
+// Check with user request
+//
+
+cout << "min_upd = " << min_upd << ", upd = " << upd << endl;
+	if ((min_upd != 0) && (upd < min_upd))
+	{
+		TangoSys_OMemStream o;
+		o << "Polling period for ";
+		if (obj_type == Tango::POLL_CMD)
+			o << "command ";
+		else
+			o << "attribute ";
+		o << obj_name << " is below the min authorized (" << min_upd << ")" << ends;
+		Except::throw_exception((const char *)"API_MethodArgument",o.str(),
+					(const char *)"DServer::check_upd_authorized");
+	}
+
+}
+
 } // End of Tango namespace
 
