@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventkeepalive.cpp,v 1.7 2009/09/22 07:55:05 jensmeyer Exp $";
+static const char *RcsId = "$Id: eventkeepalive.cpp,v 1.12 2010/10/06 12:31:15 taurel Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: eventkeepalive.cpp,v 1.7 2009/09/22 07:55:05 je
 ///
 ///		original : 7 April 2003
 //
-// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -35,9 +35,32 @@ static const char *RcsId = "$Id: eventkeepalive.cpp,v 1.7 2009/09/22 07:55:05 je
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
 ///
-///		$Revision: 1.7 $
+///		$Revision: 1.12 $
 ///
 ///		$Log: eventkeepalive.cpp,v $
+///		Revision 1.12  2010/10/06 12:31:15  taurel
+///		- Fix bug in re-connection synchronous call for attribute configuration
+///		change event
+///		
+///		Revision 1.11  2010/09/29 12:04:35  taurel
+///		- It's now possible to register several callbacks for the same event
+///		
+///		Revision 1.10  2010/09/09 13:46:00  taurel
+///		- Add year 2010 in Copyright notice
+///		
+///		Revision 1.9  2010/05/26 09:15:36  taurel
+///		- Another commit after merge with the bug fixes branch
+///		
+///		Revision 1.8  2009/12/08 07:55:15  taurel
+///		- Get some bug fixes from a merge with the Release_7_1_1-bugfixes branch
+///		Revision 1.7.2.2  2010/05/18 08:27:23  taurel
+///		- Events from device in a DS started with a file as database are now
+///		back into operation
+///		
+///		Revision 1.7.2.1  2009/12/04 15:17:50  taurel
+///		- Fix bug in case of application shutdown when the notifd is dead.
+///		If the thread abort is required, do it immediately
+///		
 ///		Revision 1.7  2009/09/22 07:55:05  jensmeyer
 ///		Changed readers to writers lock in EventConsumerKeepAliveThread::run_undetached() for connection
 ///		test of not yet connected events.
@@ -125,27 +148,40 @@ bool EventConsumerKeepAliveThread::reconnect_to_channel(EvChanIte &ipos,EventCon
 
 	for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 	{
-		if ((epos->second.channel_name == ipos->first) && 
-		   (epos->second.callback != NULL || epos->second.ev_queue != NULL))
+		if (epos->second.channel_name == ipos->first)
 		{
-			try
+			bool need_reconnect = false;
+			vector<EventSubscribeStruct>:: iterator esspos;
+			for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
 			{
-				string adm_name = ipos->second.full_adm_name;
-				event_consumer->connect_event_channel(adm_name,
-								      epos->second.device->get_device_db(),
-								      true);
-		
-				if (ipos->second.adm_device_proxy != NULL)
-					delete ipos->second.adm_device_proxy;
-				ipos->second.adm_device_proxy = new DeviceProxy(ipos->second.full_adm_name);
-				cout3 << "Reconnected to event channel" << endl;
+				if (esspos->callback != NULL || esspos->ev_queue != NULL)
+				{
+					need_reconnect = true;
+					break;
+				}
 			}
-			catch(...)
+
+			if (need_reconnect == true)
 			{
-				ret = false;
-			}
+				try
+				{
+					string adm_name = ipos->second.full_adm_name;
+					event_consumer->connect_event_channel(adm_name,
+									      epos->second.device->get_device_db(),
+									      true);
 			
-			break;
+					if (ipos->second.adm_device_proxy != NULL)
+						delete ipos->second.adm_device_proxy;
+					ipos->second.adm_device_proxy = new DeviceProxy(ipos->second.full_adm_name);
+					cout3 << "Reconnected to event channel" << endl;
+				}
+				catch(...)
+				{
+					ret = false;
+				}
+			
+				break;
+			}
 		}
 	}
 
@@ -175,29 +211,42 @@ void EventConsumerKeepAliveThread::reconnect_to_event(EvChanIte &ipos,EventConsu
 
 	for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 	{
-		if ((epos->second.channel_name == ipos->first) && 
-		   (epos->second.callback != NULL || epos->second.ev_queue != NULL))
+		if (epos->second.channel_name == ipos->first)
 		{
-			try
+			bool need_reconnect = false;
+			vector<EventSubscribeStruct>:: iterator esspos;
+			for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
+			{
+				if (esspos->callback != NULL || esspos->ev_queue != NULL)
+				{
+					need_reconnect = true;
+					break;
+				}
+			}
+
+			if (need_reconnect == true)
 			{
-				epos->second.callback_monitor->get_monitor();
-			
 				try
 				{
-					re_subscribe_event(epos,ipos);
-					epos->second.filter_ok = true;
-					cout3 << "Reconnected to event" << endl;
+					epos->second.callback_monitor->get_monitor();
+				
+					try
+					{
+						re_subscribe_event(epos,ipos);
+						epos->second.filter_ok = true;
+						cout3 << "Reconnected to event" << endl;
+					}
+					catch(...)
+					{
+						epos->second.filter_ok = false;
+					}
+				
+					epos->second.callback_monitor->rel_monitor();
 				}
-				catch(...)
+				catch (...)
 				{
-					epos->second.filter_ok = false;
+					cerr << "EventConsumerKeepAliveThread::reconnect_to_event() cannot get callback monitor for " << epos->first << endl;
 				}
-			
-				epos->second.callback_monitor->rel_monitor();
-			}
-			catch (...)
-			{
-				cerr << "EventConsumerKeepAliveThread::reconnect_to_event() cannot get callback monitor for " << epos->first << endl;
 			}
 		}
 	}
@@ -219,6 +268,7 @@ void EventConsumerKeepAliveThread::reconnect_to_event(EvChanIte &ipos,EventConsu
 
 void EventConsumerKeepAliveThread::re_subscribe_event(EvCbIte &epos,EvChanIte &ipos)
 {
+
 //
 // Build a filter using the CORBA Notify constraint Language
 // (use attribute name in lowercase letters)
@@ -228,7 +278,8 @@ void EventConsumerKeepAliveThread::re_subscribe_event(EvCbIte &epos,EvChanIte &i
   	CosNotifyFilter::Filter_var filter = CosNotifyFilter::Filter::_nil();
 	CosNotifyFilter::FilterID filter_id;
 
-	string channel_name = epos->second.channel_name;	
+	string channel_name = epos->second.channel_name;
+
 	try
 	{
    		ffp    = ipos->second.eventChannel->default_filter_factory();  
@@ -354,9 +405,12 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 				shared_cmd.cond.timedwait(s,n);
 			}
 			if (shared_cmd.cmd_pending == true)
+			{
 				exit_th = true;
+				return (void *)NULL;
+			}
 		}
-		
+
 //
 // Re-subscribe
 //
@@ -593,8 +647,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
  					bool heartbeat_skipped;
 					heartbeat_skipped = ((now - ipos->second.last_heartbeat) > (EVENT_HEARTBEAT_PERIOD + 1));
 										
-					if (heartbeat_skipped || ipos->second.heartbeat_skipped ||
-			   		ipos->second.notifd_failed == true )
+					if (heartbeat_skipped || ipos->second.heartbeat_skipped || ipos->second.notifd_failed == true )
 					{
 						ipos->second.heartbeat_skipped = true;
 
@@ -616,7 +669,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 							catch (Tango::DevFailed &)
 							{
 								// in case of failure, just stay connected to the actual notifd
-								info.server_host = ipos->second.notifyd_host;	
+								info.server_host = ipos->second.notifyd_host;
 							}
 
 							if ( ipos->second.notifyd_host != info.server_host )
@@ -624,7 +677,11 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 								ipos->second.notifd_failed = true;
 							}
 							else
+							{
 								CosNotifyChannelAdmin::EventChannelFactory_var ecf = ipos->second.eventChannel->MyFactory();
+								if (ipos->second.full_adm_name.find(MODIFIER_DBASE_NO) != string::npos)
+									ipos->second.notifd_failed = true;
+							}
 						}
 						catch (...)
 						{
@@ -675,22 +732,36 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 
 						for (epos = event_consumer->event_callback_map.begin(); epos != event_consumer->event_callback_map.end(); epos++)
 						{
-							if ((epos->second.channel_name == ipos->first) && 
-							   (epos->second.callback != NULL || epos->second.ev_queue != NULL))
+							if (epos->second.channel_name == ipos->first) 
 							{
+
+								bool need_reconnect = false;
+								vector<EventSubscribeStruct>:: iterator esspos;
+								for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
+								{
+									if (esspos->callback != NULL || esspos->ev_queue != NULL)
+									{
+										need_reconnect = true;
+										break;
+									}
+								}
+
 								// lock the callback
 								try
 								{
 									epos->second.callback_monitor->get_monitor();
 
-									if (epos->second.filter_ok == false)
+									if (need_reconnect == true)
 									{
-										try
+										if (epos->second.filter_ok == false)
 										{
-											re_subscribe_event(epos,ipos);
-											epos->second.filter_ok = true;
+											try
+											{
+												re_subscribe_event(epos,ipos);
+												epos->second.filter_ok = true;
+											}
+											catch(...) {}
 										}
-										catch(...) {}
 									}
 
 									string domain_name;
@@ -708,96 +779,99 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 										event_name = epos->first.substr(pos + 1);
 									}						
 
-									CallBack   *callback = epos->second.callback;
-									EventQueue *ev_queue = epos->second.ev_queue;
+									for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
+									{
+										CallBack   *callback = esspos->callback;
+										EventQueue *ev_queue = esspos->ev_queue;
 //
 // Push an event with error set
 //
 
-									if (event_name == CONF_TYPE_EVENT)
-									{
-										AttrConfEventData *event_data = new AttrConfEventData(epos->second.device,
-										      												domain_name,
-										      												event_name,
-										      												dev_attr_conf,
-																								errors);
-										// if a callback method was specified, call it!
-										if (callback != NULL )
+										if (event_name == CONF_TYPE_EVENT)
 										{
-											try
+											AttrConfEventData *event_data = new AttrConfEventData(epos->second.device,
+											      												domain_name,
+											      												event_name,
+											      												dev_attr_conf,
+																									errors);
+											// if a callback method was specified, call it!
+											if (callback != NULL )
 											{
-												callback->push_event(event_data);
-											}
-											catch (...)
-											{
-												cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												try
+												{
+													callback->push_event(event_data);
+												}
+												catch (...)
+												{
+													cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												}
+												
+												delete event_data;
 											}
 											
-											delete event_data;
-										}
-										
-										// no calback method, the event has to be instered
-										// into the event queue
-										else
-										{
-											ev_queue->insert_event(event_data);
-										}
-										
-									}
-									else if (event_name == DATA_READY_TYPE_EVENT)
-									{
-										DataReadyEventData *event_data = new DataReadyEventData(epos->second.device,NULL,event_name,errors);
-										// if a callback method was specified, call it!
-										if (callback != NULL )
-										{
-											try
+											// no calback method, the event has to be instered
+											// into the event queue
+											else
 											{
-												callback->push_event(event_data);
-											}
-											catch (...)
-											{
-												cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												ev_queue->insert_event(event_data);
 											}
 											
-											delete event_data;
 										}
-										
-										// no calback method, the event has to be instered
-										// into the event queue
-										else
+										else if (event_name == DATA_READY_TYPE_EVENT)
 										{
-											ev_queue->insert_event(event_data);
-										}
-									}
-									else
-									{
-										EventData *event_data = new EventData(epos->second.device,
-										      			domain_name,
-										      			event_name,
-										      			dev_attr,
-										      			errors);
-										
-										
-										// if a callback method was specified, call it!
-										if (callback != NULL )
-										{
-											try
+											DataReadyEventData *event_data = new DataReadyEventData(epos->second.device,NULL,event_name,errors);
+											// if a callback method was specified, call it!
+											if (callback != NULL )
 											{
-												callback->push_event(event_data);
-											}
-											catch (...)
-											{
-												cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												try
+												{
+													callback->push_event(event_data);
+												}
+												catch (...)
+												{
+													cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												}
+												
+												delete event_data;
 											}
 											
-											delete event_data;
+											// no calback method, the event has to be instered
+											// into the event queue
+											else
+											{
+												ev_queue->insert_event(event_data);
+											}
 										}
-										
-										// no calback method, the event has to be instered
-										// into the event queue
 										else
 										{
-											ev_queue->insert_event(event_data);
+											EventData *event_data = new EventData(epos->second.device,
+											      			domain_name,
+											      			event_name,
+											      			dev_attr,
+											      			errors);
+											
+											
+											// if a callback method was specified, call it!
+											if (callback != NULL )
+											{
+												try
+												{
+													callback->push_event(event_data);
+												}
+												catch (...)
+												{
+													cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+												}
+												
+												delete event_data;
+											}
+											
+											// no calback method, the event has to be instered
+											// into the event queue
+											else
+											{
+												ev_queue->insert_event(event_data);
+											}
 										}
 									}
 
@@ -864,35 +938,60 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 													err = e.errors;
 												}
 												epos->second.device->set_transparency_reconnection(old_transp);
-
-
-												EventData *event_data = new EventData(epos->second.device,
-													      			domain_name,
-													      			epos->second.event_name,
-													      			da,
-													      			err);
 												
-												// if a callback method was specified, call it!
-												if (callback != NULL )
+												// if callback methods were specified, call them!
+												unsigned int cb_nb = epos->second.callback_list.size();
+												unsigned int cb_ctr = 0;
+												DeviceAttribute *da_copy = NULL;
+
+												for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
 												{
-													try
+													cb_ctr++;
+													EventData *event_data;
+													if (cb_ctr != cb_nb)
 													{
-														callback->push_event(event_data);
+														da_copy = new DeviceAttribute();
+														da_copy->deep_copy(*da);
+
+														event_data = new EventData(epos->second.device,
+													      				domain_name,
+													      				epos->second.event_name,
+													      				da_copy,
+													      				err);
 													}
-													catch (...)
+													else
 													{
-														cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+														event_data = new EventData(epos->second.device,
+													      				domain_name,
+													      				epos->second.event_name,
+													      				da,
+													      				err);
+													}
+
+													CallBack   *callback = esspos->callback;
+													EventQueue *ev_queue = esspos->ev_queue;
+
+													if (callback != NULL )
+													{
+														try
+														{
+															callback->push_event(event_data);
+														}
+														catch (...)
+														{
+															cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+														}
+														
+														//event_data->attr_value = NULL;
+														delete event_data;
 													}
 													
-													//event_data->attr_value = NULL;
-													delete event_data;
-												}
-												
-												// no calback method, the event has to be instered
-												// into the event queue
-												else
-												{
-													ev_queue->insert_event(event_data);
+													// no calback method, the event has to be inserted
+													// into the event queue
+													else
+													{
+														ev_queue->insert_event(event_data);
+													}
 												}
 											}
 											
@@ -918,6 +1017,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 // The reconnection worked fine. The heartbeat should come back now,
 // when the notifd has not closed the connection.
 // Increase the counter to detect when the heartbeat is not coming back.
+
 													ipos->second.has_notifd_closed_the_connection++;		
 												}
 												catch (DevFailed &e)
@@ -926,34 +1026,57 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 												}
 												epos->second.device->set_transparency_reconnection(old_transp);
 
+												unsigned int cb_nb = epos->second.callback_list.size();
+												unsigned int cb_ctr = 0;
+												AttributeInfoEx *aie_copy = NULL;
 
-												AttrConfEventData *event_data = new AttrConfEventData(epos->second.device,
-													      			domain_name,
-													      			epos->second.event_name,
-													      			aie,
-													      			err);
-
-												// if a callback method was specified, call it!
-												if (callback != NULL )
+												for (esspos = epos->second.callback_list.begin(); esspos != epos->second.callback_list.end(); ++esspos)
 												{
-													try
+													cb_ctr++;
+													AttrConfEventData *event_data;
+													if (cb_ctr != cb_nb)
 													{
-														callback->push_event(event_data);
+														aie_copy = new AttributeInfoEx;
+														*aie_copy = *aie;
+														event_data = new AttrConfEventData(epos->second.device,
+													      				domain_name,
+													      				epos->second.event_name,
+													      				aie_copy,
+													      				err);
 													}
-													catch (...)
+													else
 													{
-														cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+														event_data = new AttrConfEventData(epos->second.device,
+													      				domain_name,
+													      				epos->second.event_name,
+													      				aie,
+													      				err);
+													}
+
+													CallBack   *callback = esspos->callback;
+													EventQueue *ev_queue = esspos->ev_queue;
+
+													// if a callback method was specified, call it!
+													if (callback != NULL )
+													{
+														try
+														{
+															callback->push_event(event_data);
+														}
+														catch (...)
+														{
+															cerr << "EventConsumerKeepAliveThread::run_undetached() exception in callback method of " << epos->first << endl;
+														}
+
+														delete event_data;
 													}
 												
-													//event_data->attr_conf = NULL;
-													delete event_data;
-												}
-												
-												// no calback method, the event has to be instered
-												// into the event queue
-												else
-												{
-													ev_queue->insert_event(event_data);
+													// no calback method, the event has to be instered
+													// into the event queue
+													else
+													{
+														ev_queue->insert_event(event_data);
+													}
 												}
 											}
 										}
