@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventsupplier.cpp,v 1.35 2009/10/27 08:25:02 taurel Exp $";
+static const char *RcsId = "$Id: eventsupplier.cpp,v 1.37 2010/09/09 13:46:00 taurel Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: eventsupplier.cpp,v 1.35 2009/10/27 08:25:02 ta
 ///
 ///		original : 29 June 2004
 ///
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -34,9 +34,23 @@ static const char *RcsId = "$Id: eventsupplier.cpp,v 1.35 2009/10/27 08:25:02 ta
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 ///
-///		$Revision: 1.35 $
+///		$Revision: 1.37 $
 ///
 ///		$Log: eventsupplier.cpp,v $
+///		Revision 1.37  2010/09/09 13:46:00  taurel
+///		- Add year 2010 in Copyright notice
+///		
+///		Revision 1.36  2010/05/26 09:15:36  taurel
+///		- Another commit after merge with the bug fixes branch
+///		
+///		Revision 1.35.2.2  2010/05/21 09:43:39  taurel
+///		- Re-use the same event channel in case of server restart when a file
+///		is use as database
+///		
+///		Revision 1.35.2.1  2010/05/18 08:27:23  taurel
+///		- Events from device in a DS started with a file as database are now
+///		back into operation
+///		
 ///		Revision 1.35  2009/10/27 08:25:02  taurel
 ///		- No real changes. Only some code beautifulling
 ///		
@@ -286,7 +300,10 @@ EventSupplier *EventSupplier::create(CORBA::ORB_var _orb,
 		new EventSupplier(_orb,ns.SupAdm,ns.pID,ns.ProCon,ns.StrProPush,ns.EveChaFac,ns.EveCha,ns.ec_ior);
 		
 	_event_supplier->fqdn_prefix = "tango://";
-	_event_supplier->fqdn_prefix = _event_supplier->fqdn_prefix + db->get_db_host() + ':' + db->get_db_port() + '/' ;
+	if (Util::_FileDb == true)
+		_event_supplier->fqdn_prefix = _event_supplier->fqdn_prefix + host_name + ':';
+	else
+		_event_supplier->fqdn_prefix = _event_supplier->fqdn_prefix + db->get_db_host() + ':' + db->get_db_port() + '/' ;
 	
 	return _event_supplier;
 }
@@ -440,8 +457,7 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 // reference so we can invoke its methods
 //
 
-		_eventChannelFactory =
-	    	CosNotifyChannelAdmin::EventChannelFactory::_narrow(event_factory_obj);
+		_eventChannelFactory = CosNotifyChannelAdmin::EventChannelFactory::_narrow(event_factory_obj);
 
 //
 // Make sure the CORBA object was really an EventChannelFactory
@@ -539,7 +555,21 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 		}
 	}
 	else
-		channel_exported = false;
+	{
+		try
+		{
+			Tango::DbDatum na;
+		
+			string cl_name(NOTIFD_CHANNEL);
+			na = db->get_device_name(server_name,cl_name);
+			channel_ior = na.value_string[0];
+			channel_exported = 1;
+		}
+		catch (Tango::DevFailed &)
+		{
+			channel_exported = 0;						
+		}
+	}
 
 	if (channel_exported)
 	{
@@ -622,10 +652,16 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 			
 //
 // In case of DS started with -file option, store the
-// event channel within the event supplier object
+// event channel within the event supplier object and in the file
 //
 				
 				ns.ec_ior = ior_string;
+
+				try
+				{
+					db->write_event_channel_ior_filedatabase(server_name,ior_string);
+				}
+				catch (Tango::DevFailed &e) {}
 			}
 			CORBA::string_free(_ior);
 		}
@@ -647,6 +683,8 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 	else
 	{
 		cout4 << "Tango::EventSupplier::create(): _narrow worked, use this event channel\n";
+		if (Util::_FileDb == true)
+			ns.ec_ior = channel_ior;
 	}
 
 //
@@ -657,8 +695,7 @@ void EventSupplier::connect_to_notifd(NotifService &ns,CORBA::ORB_var &_orb,
 // We'll use the channel's default Supplier admin
 //
 
-	CosNotifyChannelAdmin::SupplierAdmin_var 
-		_supplierAdmin = _eventChannel -> default_supplier_admin();
+	CosNotifyChannelAdmin::SupplierAdmin_var _supplierAdmin = _eventChannel -> default_supplier_admin();
 	if (CORBA::is_nil(_supplierAdmin))
 	{
         cerr << "Could not get CosNotifyChannelAdmin::SupplierAdmin" << endl;
@@ -2762,5 +2799,27 @@ void EventSupplier::disconnect_from_notifd()
 	catch(CORBA::Exception &) {}
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		EventSupplier::set_svr_port_num()
+// 
+// description : 	In case of DS using file as database, add the server port
+//					number received as argument to the fqdn prefix
+//
+// argument : in :	port_num : The DS port number
+//
+//-----------------------------------------------------------------------------
+
+void EventSupplier::set_svr_port_num(string &port_num)
+{
+
+//
+// If the port number is not already specified in the fqdn prefix, add it
+//
+
+	unsigned int sz = _instance->fqdn_prefix.size();
+	if (_instance->fqdn_prefix[sz - 1] == ':')
+		_instance->fqdn_prefix = _instance->fqdn_prefix + port_num + "/#";
+}
 
 } /* End of Tango namespace */
