@@ -30,9 +30,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 1.3 $
+// $Revision: 1.4 $
 //
 // $Log: jpeg_decoder.cpp,v $
+// Revision 1.4  2009/11/02 08:36:17  taurel
+// - Fix warnings reported when compiling using the option -Wall
+//
 // Revision 1.3  2009/04/20 14:55:58  jlpons
 // Added GPL header, changed memory allocation to C++ fashion.
 //
@@ -699,7 +702,6 @@ int jpeg_decode(int jpegSize,unsigned char *jpegData,
   errCode = jpeg_decoder_init(&decoder);ERROR(errCode);
 
   int nbMCU   = decoder.mcuNbCol * decoder.mcuNbRow;
-  int nbBlock = nbMCU * decoder.mcuNbBlock;
   int rWidth  = decoder.mcuNbRow * decoder.mcuWidth;
   int rHeight = decoder.mcuNbCol * decoder.mcuHeight;
   int mcuSize = decoder.mcuNbBlock*64;
@@ -774,7 +776,6 @@ int jpeg_decode(int jpegSize,unsigned char *jpegData,
   // Clip and Copy frame
   int rPitch;
   int rowSize;
-  int rowSizeL = decoder.width;
 
   *width = decoder.width;
   *height = decoder.height;
