@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: logging.cpp,v 3.14 2009/04/07 15:23:36 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: logging.cpp,v 3.16 2010/09/09 13:46:00 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+=============================================================================
 //
@@ -10,7 +10,7 @@ static const char *RcsId = "$Id: logging.cpp,v 3.14 2009/04/07 15:23:36 taurel E
 //
 // author(s) :  N.Leclercq - SOLEIL
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -30,9 +30,16 @@ static const char *RcsId = "$Id: logging.cpp,v 3.14 2009/04/07 15:23:36 taurel E
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.14 $
+// $Revision: 3.16 $
 //
 // $Log: logging.cpp,v $
+// Revision 3.16  2010/09/09 13:46:00  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.15  2010/06/18 07:45:47  taurel
+// - In case of locked device, polling and logging related commands are
+// allowed only for the locker process
+//
 // Revision 3.14  2009/04/07 15:23:36  taurel
 // - Fix some warnings when compiled using gcc 4.3 on 64 bits computer
 //
@@ -390,6 +397,11 @@ void Logging::add_logging_target (const Tango::DevVarStringArray *argin)
         Except::throw_exception((const char *)"API_DeviceNotFound",o.str(),
                                 (const char *)"Logging::add_logging_target");
       }
+	  // Check that none of the concerned device(s) is locked by another client
+	  DServer *adm_dev = Util::instance()->get_dserver_device();
+	  for (unsigned int j = 0; j < dl.size(); j++) {
+		adm_dev->check_lock_owner(dl[j],"add_logging_target",(dl[j]->get_name()).c_str());
+	  }
       // for each device matching pattern...
       for (unsigned int j = 0; j < dl.size(); j++) {
         // ...add logging target
@@ -658,6 +670,11 @@ void Logging::remove_logging_target (const Tango::DevVarStringArray *argin)
               Except::throw_exception((const char *)"API_DeviceNotFound",o.str(),
                                       (const char *)"Logging::remove_logging_target");
       }
+	  // Check that none of the concerned device(s) is locked by another client
+	  DServer *adm_dev = Util::instance()->get_dserver_device();
+	  for (unsigned int j = 0; j < dl.size(); j++) {
+		adm_dev->check_lock_owner(dl[j],"remove_logging_target",(dl[j]->get_name()).c_str());
+	  }
       // get target type and name from argin (syntax type::name)
       type_name = (*argin)[i++];
       std::transform(type_name.begin(), type_name.end(), type_name.begin(), ::tolower);
@@ -854,6 +871,11 @@ void Logging::set_logging_level (const DevVarLongStringArray *argin)
       std::transform(pattern.begin(), pattern.end(), pattern.begin(), ::tolower);
       // get devices which name matches the pattern pattern
       dl = Util::instance()->get_device_list(pattern);
+	  // Check that none of the concerned device(s) is locked by another client
+	  DServer *adm_dev = Util::instance()->get_dserver_device();
+	  for (unsigned int j = 0; j < dl.size(); j++) {
+		adm_dev->check_lock_owner(dl[j],"set_logging_level",(dl[j]->get_name()).c_str());
+	  }
       // for each device in dl
       for (unsigned int j = 0; j < dl.size(); j++) {
         // get device's logger (created if does not already exist)
