@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollring.cpp,v 3.24 2009/11/09 12:04:31 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: pollring.cpp,v 3.26 2010/09/09 13:46:01 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+============================================================================
 //
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: pollring.cpp,v 3.24 2009/11/09 12:04:31 taurel
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -34,9 +34,19 @@ static const char *RcsId = "$Id: pollring.cpp,v 3.24 2009/11/09 12:04:31 taurel
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.24 $
+// $Revision: 3.26 $
 //
 // $Log: pollring.cpp,v $
+// Revision 3.26  2010/09/09 13:46:01  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.25  2009/12/08 07:55:15  taurel
+// - Get some bug fixes from a merge with the Release_7_1_1-bugfixes branch
+//
+// Revision 3.24.2.1  2009/12/07 14:17:29  taurel
+// - Correctly nitialized the write part  y dimension of spectrum or image
+// attribute when retrieving the attribute history from the polling buffer
+//
 // Revision 3.24  2009/11/09 12:04:31  taurel
 // - The attribute mutex management is in the AttributeValue_4 struct
 //
@@ -2216,7 +2226,7 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistory_4 *ptr,long type)
 			int r_dim_x = (*ring[index].attr_value_4)[0].r_dim.dim_x;
 			int r_dim_y = (*ring[index].attr_value_4)[0].r_dim.dim_y;
 			int w_dim_x = (*ring[index].attr_value_4)[0].w_dim.dim_x;
-			int w_dim_y = (*ring[index].attr_value_4)[0].w_dim.dim_x;
+			int w_dim_y = (*ring[index].attr_value_4)[0].w_dim.dim_y;
 
 			int data_length;
 			(r_dim_y == 0) ? data_length = r_dim_x : data_length = r_dim_x * r_dim_y;
