@@ -11,7 +11,7 @@
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -31,9 +31,16 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.10 $
+// $Revision: 3.12 $
 //
 // $Log: pollring.h,v $
+// Revision 3.12  2010/09/09 13:46:01  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.11  2010/09/09 13:29:09  taurel
+// - Commit after the last merge with the bugfixes branch
+// - Fix some warning when compiled -W -Wall
+//
 // Revision 3.10  2009/09/18 09:18:06  taurel
 // - End of attribute serialization implementation?
 //
@@ -207,12 +214,12 @@ public:
 	struct timeval				when;
 };
 
-inline bool operator<(const RingElt &l,const RingElt &r)
+inline bool operator<(const RingElt &,const RingElt &)
 {
 	return true;
 }
 
-inline bool operator==(const RingElt &l,const RingElt &r)
+inline bool operator==(const RingElt &,const RingElt &)
 {
 	return true;
 }
