@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollthread.cpp,v 3.27 2009/10/23 14:36:27 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: pollthread.cpp,v 3.30 2010/09/09 13:46:45 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+============================================================================
 //
@@ -13,7 +13,7 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.27 2009/10/23 14:36:27 taure
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -33,9 +33,22 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.27 2009/10/23 14:36:27 taure
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.27 $
+// $Revision: 3.30 $
 //
 // $Log: pollthread.cpp,v $
+// Revision 3.30  2010/09/09 13:46:45  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.29  2010/09/08 12:32:53  taurel
+// - Miscellaneous changes to implement a better timeout management
+// (now manage a user connect timeout with the env. variable TANGOconnectTimeout)
+//
+// Revision 3.28  2010/01/20 07:53:02  taurel
+// - Commit after merge with the Release_7_1_1-bugfixes branch
+//
+// Revision 3.27.2.1  2010/01/18 16:12:40  taurel
+// - Fix polling thread tuning algorithum bug in case of object with very slow polling period (> 100 sec)
+//
 // Revision 3.27  2009/10/23 14:36:27  taurel
 // - Tango 7.1.1
 // - Fix bugs 2880372 and 2881841
@@ -454,7 +467,17 @@ void *PollThread::run_undetached(void *ptr)
 		}
 		catch (omni_thread_fatal &)
 		{
-			cerr << "OUPS !! A omni thread fatal exception !!!!!!!!" << endl;
+			cerr << "OUPS !! A omni thread fatal exception received by a polling thread !!!!!!!!" << endl;
+#ifndef _TG_WINDOWS_
+			time_t t = time(NULL);
+			cerr << ctime(&t);
+#endif
+			cerr << "Trying to re-enter the main loop" << endl;
+		}
+		catch (const std::exception &ex)
+		{
+			cerr << "OUPS !! An unforeseen standard exception has been received by a polling thread !!!!!!" << endl;
+			cerr << ex.what() << endl;
 #ifndef _TG_WINDOWS_
 			time_t t = time(NULL);
 			cerr << ctime(&t);
@@ -1200,7 +1223,7 @@ void PollThread::tune_list(bool from_needed, long min_delta)
 //
 
 	unsigned long needed_sum = 0;
-	unsigned long min_upd = 100000000;
+	unsigned long min_upd = 0;
 	long max_delta_needed;
 	
 	if (from_needed == true)
@@ -1211,8 +1234,16 @@ void PollThread::tune_list(bool from_needed, long min_delta)
 			needed_sum = needed_sum + (unsigned long)needed_time_usec;
 			
 			unsigned long update_usec = (unsigned long)ite->update * 1000;
-			if (min_upd > update_usec)
+			
+			if (ite == works.begin())
+			{
 				min_upd = update_usec;
+			}
+			else
+			{
+				if (min_upd > update_usec)
+					min_upd = update_usec;
+			}
 		}
 
 //
