@@ -1,32 +1,26 @@
 // -*- Mode: C++; -*-
-//                          Package   : omniNames
+//
 // ReadersWritersLock.h     Author    : Tristan Richardson (tjr)
 //
-//    Copyright (C) 1997-1999 AT&T Laboratories Cambridge
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
 //
-//  This file is part of omniNames.
-//
-//  omniNames is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU Lesser General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-//
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU Lesser General Public License for more details.
-//
-//  You should have received a copy of the GNU Lesser General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
-//  USA.
-//
-
-//
-// This class implements a multiple-readers, single-writer lock.  It has
-// a slight alteration from a standard such lock.  Any thread which has already
-// called writerIn() may call writerIn() or readerIn() again multiple times.
+// This file is part of Tango.
 //
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 
 
 #ifndef _ReadersWritersLock_h_
@@ -37,70 +31,78 @@
 class ReadersWritersLock {
 
 public:
-  omni_mutex m;
-  omni_condition c;
+  omni_mutex mut;
+  omni_condition cond;
   int n;	// 0 means no-one active, > 0 means n readers, < 0 means writer
 		// (-n times).
   int writerId;
 
-  ReadersWritersLock(void) : c(&m), n(0), writerId(0) {}
+  ReadersWritersLock(void) : cond(&mut), n(0), writerId(0), auto_self(NULL) {}
 
-  void readerIn(void) {
-    m.lock();
+  void readerIn(void)
+  {
+    mut.lock();
 	 
 	 // In the case of usage with another threading library, omni_thread::self() might
 	 // return a NULL pointer!
-	 int threadId = 0;	 
-	 if ( omni_thread::self() != NULL )
-		{
-		threadId = omni_thread::self()->id();
-		}
+	 int threadId = 0;
+	 omni_thread *th = omni_thread::self(); 
+	 if ( th != NULL )
+	 {
+		threadId = th->id();
+	 }
 	 	 
-    if ((n < 0) && (writerId == threadId)) {
+     if ((n < 0) && (writerId == threadId))
+	 {
       // this thread already has lock as writer, simply decrement n
       n--;
-      m.unlock();
+      mut.unlock();
       return;
-    }
-    while (n < 0)
-      c.wait();
-    n++;
-    m.unlock();
+     }
+     while (n < 0)
+       cond.wait();
+     n++;
+     mut.unlock();
   }
 
-  void readerOut(void) {
-    m.lock();
-    if (n < 0) {
+  void readerOut(void)
+  {
+    mut.lock();
+    if (n < 0)
+	{
       // this thread already had lock as writer, simply increment n
       n++;
-      m.unlock();
+      mut.unlock();
       return;
     }
     n--;
     if (n == 0)
-      c.signal();
-    m.unlock();
+      cond.signal();
+    mut.unlock();
   }
 
-  void writerIn(void) {
-    m.lock();
+  void writerIn(void)
+  {
+    mut.lock();
 	 
 	 // In the case of usage with another threading library, omni_thread::self() might
 	 // return a NULL pointer! 
-	 int threadId = 0;	 
-	 if ( omni_thread::self() != NULL )
-		{
-		threadId = omni_thread::self()->id();
-		}			 
+	 int threadId = 0;
+	 omni_thread *th = omni_thread::self(); 
+	 if ( th != NULL )
+	 {
+		threadId = th->id();
+	 }			 
 			 
-    if ((n < 0) && (writerId == threadId)) {
+     if ((n < 0) && (writerId == threadId))
+	 {
       // this thread already has lock as writer, simply decrement n
       n--;
-      m.unlock();
+      mut.unlock();
       return;
-    }
-    while (n != 0)
-      c.wait();
+     }
+     while (n != 0)
+       cond.wait();
     
 	 n--;
 	 
@@ -108,31 +110,37 @@ public:
 	 // Make sure we get a correct thread ID
 	 // With the class ensure_self it should return always a thread ID.
 	 // Create the ensure_self object only for the thread which takes the writer lock!
-	 auto_self = new omni_thread::ensure_self();
+	 if (th == NULL)
+	 	auto_self = new omni_thread::ensure_self();
 	 writerId  = omni_thread::self()->id();
 	 	 
-    m.unlock();
+     mut.unlock();
   }
 
-  void writerOut(void) {
-    m.lock();
+  void writerOut(void)
+  {
+    mut.lock();
     n++;
     if (n == 0)
-	 	{
+	{
 		// delete the dummy thread when it was created.
-		delete auto_self;
-      
-		c.broadcast();	// might as well wake up all readers
+		if (auto_self != NULL)
+		{
+			delete auto_self;
+			auto_self = NULL;
 		}
-    m.unlock();
+      
+		cond.broadcast();	// might as well wake up all readers
+	}
+    mut.unlock();
   }
-
+  
 private:
-	 // in the case of usage with another threading library, omni_thread::self() might
-	 // return a NULL pointer!
+	// in the case of usage with another threading library, omni_thread::self() might
+	// return a NULL pointer!
     // To avoid this problem we use the class ensure_self to get a dummy thread ID!
-	 //
-	 // The class ensure_self should be created on the stack. If created in
+	//
+	// The class ensure_self should be created on the stack. If created in
     // a thread without an associated omni_thread, it creates a dummy
     // thread which is released when the ensure_self object is deleted.
 
