@@ -8,7 +8,7 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -28,9 +28,26 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.49 $
+// $Revision: 3.53 $
 //
 // $Log: tango_const.h,v $
+// Revision 3.53  2010/09/09 13:46:45  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.52  2010/09/07 15:32:21  taurel
+// - Fix some re-connection problems with Windows
+// - Publish all endPoints in case of multiple network interface
+//
+// Revision 3.51  2010/08/19 12:10:20  taurel
+// - Change timeout during the first _narrow() call in connect() method
+//
+// Revision 3.50  2010/05/26 09:15:36  taurel
+// - Another commit after merge with the bug fixes branch
+//
+// Revision 3.49.2.1  2010/05/21 09:43:39  taurel
+// - Re-use the same event channel in case of server restart when a file
+// is use as database
+//
 // Revision 3.49  2009/10/23 14:36:27  taurel
 // - Tango 7.1.1
 // - Fix bugs 2880372 and 2881841
@@ -154,7 +171,7 @@ namespace Tango
 // Some general interest define
 //
 
-#define		TgLibVers				"7.1.1"		// Please, always code this following format "X.Y.Z"
+#define		TgLibVers				"7.2.0"		// Please, always code this following format "X.Y.Z"
 
 #define		DevVersion				4			// IDL version number
 #define		DefaultMaxSeq			20
@@ -209,6 +226,12 @@ namespace Tango
 #define     DELTA_PERIODIC_LONG			100   // For times > 5000ms only keep a delta of 100ms
 #define		HEARTBEAT					"Event heartbeat"
 
+//
+// Event when using a file as database stuff
+//
+
+#define		NOTIFD_CHANNEL				"notifd_channel"
+
 //
 // Locking feature related defines
 //
@@ -224,6 +247,7 @@ namespace Tango
 
 #define		CLNT_TIMEOUT_STR		"3000"
 #define		CLNT_TIMEOUT			3000
+#define		NARROW_CLNT_TIMEOUT		100
 
 //
 // Connection and call timeout for database device
