@@ -8,7 +8,7 @@
 //
 // author(s) :          E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -28,9 +28,20 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.8 $
+// $Revision: 3.11 $
 //
 // $Log: tango_monitor.h,v $
+// Revision 3.11  2010/09/09 13:46:45  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.10  2010/01/07 08:35:51  taurel
+// - Several change sto improve thread safety of the DeviceProxy, AttributeProxy, ApiUtul and EventConsumer classes
+//
+// Revision 3.9  2009/12/18 14:52:37  taurel
+// - Safety commit before christmas holydays
+// - Many changes to make the DeviceProxy, Database and AttributeProxy
+// classes thread safe (good help from the helgrind tool from valgrind)
+//
 // Revision 3.8  2009/01/21 12:49:03  taurel
 // - Change CopyRights for 2009
 //
@@ -101,15 +112,8 @@ public :
 	
 	void signal() {cond.signal();}
 	
-	int get_locking_thread_id()
-	{
-	if (locking_thread != NULL)
-		return locking_thread->id();
-	else
-		return 0;
-	}
-	
-	long get_locking_ctr() {return locked_ctr;}
+	int get_locking_thread_id();	
+	long get_locking_ctr();
 			
 private :
 	long 			_timeout;
@@ -120,6 +124,27 @@ private :
 };
 
 
+//+-------------------------------------------------------------------------
+//
+// methods : 		TangoMonitor::get_locking_thread_id
+//					TangoMonitor::get_locking_ctr
+//
+//--------------------------------------------------------------------------
+
+
+inline int TangoMonitor::get_locking_thread_id()
+{
+	if (locking_thread != NULL)
+		return locking_thread->id();
+	else
+		return 0;
+}
+
+inline long TangoMonitor::get_locking_ctr()
+{
+	omni_mutex_lock guard(*this);
+	return locked_ctr;
+}
 
 //+-------------------------------------------------------------------------
 //
