@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.61 2009/10/26 13:49:54 taurel Exp $\n$Name: Release_7_1_1 $";
+static const char *RcsId = "$Id: utils.cpp,v 3.68 2010/09/09 13:46:45 taurel Exp $\n$Name: Release_7_2_0 $";
 
 //+=============================================================================
 //
@@ -11,7 +11,7 @@ static const char *RcsId = "$Id: utils.cpp,v 3.61 2009/10/26 13:49:54 taurel Exp
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -31,9 +31,44 @@ static const char *RcsId = "$Id: utils.cpp,v 3.61 2009/10/26 13:49:54 taurel Exp
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.61 $
+// $Revision: 3.68 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.68  2010/09/09 13:46:45  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.67  2010/09/08 08:20:44  taurel
+// = Change the way the ORB is initialised to better manage several network
+// interface host case
+//
+// Revision 3.66  2010/09/07 15:32:21  taurel
+// - Fix some re-connection problems with Windows
+// - Publish all endPoints in case of multiple network interface
+//
+// Revision 3.65  2010/05/26 09:15:36  taurel
+// - Another commit after merge with the bug fixes branch
+//
+// Revision 3.64  2010/01/20 08:00:47  taurel
+// - Fix compilation problem introduced by the previous merge
+// Revision 3.61.2.3  2010/05/18 08:27:23  taurel
+// - Events from device in a DS started with a file as database are now
+// back into operation
+//
+// Revision 3.63  2010/01/20 07:53:02  taurel
+// - Commit after merge with the Release_7_1_1-bugfixes branch
+//
+// Revision 3.62  2009/12/18 14:52:37  taurel
+// - Safety commit before christmas holydays
+// - Many changes to make the DeviceProxy, Database and AttributeProxy
+// classes thread safe (good help from the helgrind tool from valgrind)
+// Revision 3.61.2.2  2009/12/18 14:49:02  taurel
+// - If the DS main thread is not created using omnithread, it is also
+// necessary to call omnithread::release_dummy() before the thread exit
+//
+// Revision 3.61.2.1  2009/12/18 14:07:34  taurel
+// - Add omni_thread call in case the main DS thread is not the process
+// main thread.
+//
 // Revision 3.61  2009/10/26 13:49:54  taurel
 // - Add an exception case in the Util::get_device_list_by_class() method
 //
@@ -343,6 +378,29 @@ void Util::effective_job(int argc,char *argv[])
 
 		DServerSignal::instance();
 
+//
+// Check if the user specified a endPoint on the command line or using one
+// env. variable
+//
+
+		bool endpoint_specified = false;
+		for (int i = 2;i < argc;i++)
+		{
+			if (::strcmp("-ORBendPoint",argv[i]) == 0)
+			{
+				endpoint_specified = true;
+				break;
+			}
+		}
+
+		if (endpoint_specified == false)
+		{
+			DummyDeviceProxy d;
+			string env_var;
+			if (d.get_env_var("ORBendPoint",env_var) == 0)
+				endpoint_specified = true;
+		}
+
 //
 // Initialise CORBA ORB
 //
@@ -353,48 +411,46 @@ void Util::effective_job(int argc,char *argv[])
 		WSAStartup(rel,&dat);
 #endif
 
-		char h_name[80];
-		int res = gethostname(h_name,80);
-		if (res == 0)
+		if (endpoint_specified == true)
 		{
 			const char *options[][2] = {
-			{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
-			{"serverCallTimeOutPeriod","5000"},
-			{"maxServerThreadPoolSize","100"},
-			{"threadPerConnectionUpperLimit","55"},
-			{"threadPerConnectionLowerLimit","50"},
-			{"supportCurrent","0"},
-			{"verifyObjectExistsAndType","0"},
-			{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
+				{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
+				{"serverCallTimeOutPeriod","5000"},
+				{"maxServerThreadPoolSize","100"},
+				{"threadPerConnectionUpperLimit","55"},
+				{"threadPerConnectionLowerLimit","50"},
+				{"supportCurrent","0"},
+				{"verifyObjectExistsAndType","0"},
+				{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
 #ifndef _TG_WINDOWS_
-			{"endPoint","giop:tcp::"},
-			{"endPoint","giop:unix:"},
+				{"endPoint","giop:unix:"},
 #endif
-			{0,0}};
+				{0,0}
+			};
 			
 			orb = CORBA::ORB_init(argc,argv,"omniORB4",options);
 		}
 		else
 		{
 			const char *options[][2] = {
-			{"endPointPublish","all(addr)"},
-			{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
-			{"serverCallTimeOutPeriod","5000"},
-			{"maxServerThreadPoolSize","100"},
-			{"threadPerConnectionUpperLimit","55"},
-			{"threadPerConnectionLowerLimit","50"},
-			{"supportCurrent","0"},
-			{"verifyObjectExistsAndType","0"},
-			{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
+				{"endPointPublish","all(addr)"},
+				{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
+				{"serverCallTimeOutPeriod","5000"},
+				{"maxServerThreadPoolSize","100"},
+				{"threadPerConnectionUpperLimit","55"},
+				{"threadPerConnectionLowerLimit","50"},
+				{"supportCurrent","0"},
+				{"verifyObjectExistsAndType","0"},
+				{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
 #ifndef _TG_WINDOWS_
-			{"endPoint","giop:tcp::"},
-			{"endPoint","giop:unix:"},
+				{"endPoint","giop:tcp::"},
+				{"endPoint","giop:unix:"},
 #endif
-			{0,0}};
+				{0,0}
+			};
 			
 			orb = CORBA::ORB_init(argc,argv,"omniORB4",options);
 		}
-		
 
 #ifndef TANGO_HAS_LOG4TANGO
 
@@ -784,6 +840,33 @@ void Util::check_args(int argc,char *argv[])
 #endif
 						cout4 << "File name = <" << database_file_name << ">" << endl;
 						ind++;
+
+//
+// Try to find the ORB endPoint option
+//
+
+						long arg_nb;
+						for (arg_nb = 2;arg_nb < argc;arg_nb++)
+						{
+							if (::strcmp(argv[arg_nb],"-ORBendPoint") == 0)
+							{
+								arg_nb++;
+								string endpoint = argv[arg_nb];
+								string::size_type pos;
+								if ((pos = endpoint.rfind(':')) == string::npos)
+								{
+									cerr << "Strange ORB endPoint specification" << endl;
+									print_usage(argv[0]);
+								}
+								ext->svr_port_num = endpoint.substr(++pos);
+								break;
+							}
+						}
+						if (arg_nb == argc)
+						{
+							cerr << "Missing ORB endPoint specification" << endl;
+							print_usage(argv[0]);
+						}
 					}
 					break;
 
@@ -1368,9 +1451,20 @@ void Util::create_event_supplier()
 			ext->event_supplier = EventSupplier::create(orb,ds_name,db,hostname,this);
 			ext->event_supplier->connect();
 		}
+		catch (Tango::DevFailed &e)
+		{
+			ext->event_supplier = NULL;
+			if (_FileDb == true)
+			{
+				Tango::Except::print_exception(e);
+				cerr << "Can't create event supplier. Event not available" << endl;
+			}
+		}
 		catch (...)
 		{
 			ext->event_supplier = NULL;
+			if (_FileDb == true)
+				cerr << "Can't create event supplier. Event not available" << endl;
 		}
 	}
 	else
@@ -1611,10 +1705,24 @@ void Util::server_init(bool with_window)
 	if (Util::_service == true)
 	{
 		omni_thread::create_dummy();
+		ext->_dummy_thread = true;
+	}
+	
+	omni_thread *th = omni_thread::self();
+	if (th == NULL)
+	{
+		th = omni_thread::create_dummy();
+		ext->_dummy_thread = true;
+	}
+#else
+	omni_thread *th = omni_thread::self();
+	if (th == NULL)
+	{
+		th = omni_thread::create_dummy();
+		ext->_dummy_thread = true;
 	}
 #endif
-
-	omni_thread *th = omni_thread::self();
+	
 	if (is_py_ds() == false)
 	{
 		th->set_value(key_py_data,new Tango::PyData());
@@ -1887,7 +1995,10 @@ void Util::server_cleanup()
 			_constructed = false;
 		}
 	}
-#endif	
+#endif
+
+	if (ext->_dummy_thread == true)
+		omni_thread::release_dummy();
 }
 
 //+----------------------------------------------------------------------------
