@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: utils_polling.cpp,v 3.12 2009/11/10 12:08:28 taurel Exp $";
+static const char *RcsId = "$Id: utils_polling.cpp,v 3.15 2010/09/09 13:46:45 taurel Exp $";
 
 //+=============================================================================
 //
@@ -11,7 +11,7 @@ static const char *RcsId = "$Id: utils_polling.cpp,v 3.12 2009/11/10 12:08:28 ta
 //
 // author(s) :         	E.Taurel
 //
-// Copyright (C) :      2004,2005,2006,2007,2008,2009
+// Copyright (C) :      2004,2005,2006,2007,2008,2009,2010
 //						European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -31,9 +31,18 @@ static const char *RcsId = "$Id: utils_polling.cpp,v 3.12 2009/11/10 12:08:28 ta
 // You should have received a copy of the GNU Lesser General Public License
 // along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
-// $Revision: 3.12 $
+// $Revision: 3.15 $
 //
 // $Log: utils_polling.cpp,v $
+// Revision 3.15  2010/09/09 13:46:45  taurel
+// - Add year 2010 in Copyright notice
+//
+// Revision 3.14  2010/06/21 12:47:12  taurel
+// - Add some logging message in polling_configure method
+//
+// Revision 3.13  2010/04/27 07:36:42  taurel
+// - Merge with the bugfixes branch
+//
 // Revision 3.12  2009/11/10 12:08:28  taurel
 // - Change the default value for the UtilExt::poll_pool_size data member
 //
@@ -98,7 +107,8 @@ namespace Tango
 
 void Util::polling_configure()
 {
-
+	cout4 << "Entering polling_configure()" << endl;
+	
 //
 // Get the polling threads pool conf from the DServer device
 //
@@ -193,7 +203,6 @@ void Util::polling_configure()
 
 			for (k = 0;k < poll_cmd_list.size();k++)
 			{
-
 				DevVarLongStringArray *send = new DevVarLongStringArray();
 				send->lvalue.length(1);
 				send->svalue.length(3);
@@ -232,7 +241,6 @@ void Util::polling_configure()
 
 			for (k = 0;k < poll_attr_list.size();k++)
 			{
-
 				DevVarLongStringArray *send = new DevVarLongStringArray();
 				send->lvalue.length(1);
 				send->svalue.length(3);
@@ -378,6 +386,8 @@ void Util::polling_configure()
 	
 	if (((dev_db_upd.empty() == false) || (ext->conf_needs_db_upd == true)) && (_UseDb == true))
 		upd_polling_prop(dev_db_upd,admin_dev);
+		
+	cout4 << "Leaving polling_configure()" << endl;
 
 }
 
@@ -827,7 +837,7 @@ int Util::create_poll_thread(const char *dev_name,bool startup,int smallest_upd)
 //
 // Create a new polling thread and start it
 //
-		
+
 		PollingThreadInfo *pti_ptr = new PollingThreadInfo();
 		if (smallest_upd != -1)
 			pti_ptr->smallest_upd = smallest_upd;
@@ -944,7 +954,7 @@ int Util::create_poll_thread(const char *dev_name,bool startup,int smallest_upd)
 //
 // Assign this device to the thread which "seems" to have the less work
 //
-		
+
 		(*lower_iter)->polled_devices.push_back(local_dev_name);
 		ext->dev_poll_th_map.insert(make_pair(local_dev_name,(*lower_iter)->thread_id));
 		if (smallest_upd != -1)
@@ -1032,7 +1042,7 @@ void Util::stop_heartbeat_thread()
 //
 // method : 		Util::get_poll_th_id_by_name()
 // 
-// description : 	Get the ID of the thread in charg of the device polling
+// description : 	Get the ID of the thread in charge of the device polling
 //
 // argin() : dev_name : The device name
 //
